/**
* Matthew Netsch
* Feb 2010
*
* MainWindow.h
*
* Contains all of the tabs
* for the program. This also
* acts as a relay for all of the
* messages that need to be
* passed between group boxes.
*/

#include "MainWindow.h"

/* Constructor */
MainWindow::MainWindow()
{
    statusLabel = new QLabel(tr(""));
    tempLabel = new QLabel(tr(""));
    progressLabel = new QLabel(tr(""));
    hintLabel = new QLabel(tr(""));
    progressBar = new QProgressBar();

    for(int i = 0; i < 5; i++)
      statusBarPos[i] = -1;

    progressBar->setMaximum(100);
    progressBar->setMinimum(0);
    progressBar->setValue(0);


    cancelClose = false;
    createControllers();

    createTabs();
    createMenus();

    /*Set Title and Icons*/
    setWindowTitle(tr("Neural Network TAC"));

    QIcon icon;
    icon.addFile(QString::fromUtf8(":/new/icons/TAC.ico"), QSize(), QIcon::Normal, QIcon::Off);
    setWindowIcon(icon);

    /*Set Size Policy*/
    this->setMinimumSize(800, 484);
    resize(930, 600);

    setAutoFillBackground(true);

    /*Font*/
    QFont font;
    font.setBold(true);
    font.setWeight(75);
    setFont(font);

    QFont statusFont;
    statusFont.setPointSize(10);
    statusLabel->setFont(statusFont);
    tempLabel->setFont(statusFont);
    progressLabel->setFont(statusFont);

    statusFont.setBold(true);
    hintLabel->setFont(statusFont);

    setCentralWidget(TabWidget);
    
    statusChanged();

}

/* Create all of the tabs */
void MainWindow::createTabs()
{
    /* Tab Widget properties */
    TabWidget = new QTabWidget();

    TabWidget->setTabShape(QTabWidget::Rounded);
    TabWidget->setIconSize(QSize(32, 32));
    TabWidget->setElideMode(Qt::ElideNone);

    TabWidget->setAutoFillBackground(true);

    DeviceTab = new DeviceViewTab();
    connect(this, SIGNAL(deviceChanged(NNDevice*)), DeviceTab, SLOT(deviceChanged(NNDevice*)));
    connect(DeviceTab, SIGNAL(statusChanged()), this, SLOT(statusChanged()));

    NetworkTab = new NetworkViewTab();
    connect(this, SIGNAL(networkOpened(NNApprox*,QString*)), NetworkTab, SIGNAL(networkOpened(NNApprox*,QString*)));
    connect(this, SIGNAL(networkUpdated(NNApprox*)), NetworkTab, SIGNAL(networkUpdated(NNApprox*)));
    connect(NetworkTab, SIGNAL(updateNetwork(NNApprox*)), this, SIGNAL(networkUpdated(NNApprox*)));
    connect(NetworkTab, SIGNAL(statusChanged()), this, SLOT(statusChanged()));
    connect(this, SIGNAL(changedFilename(QString*)), NetworkTab, SLOT(changedFilename(QString*)));

    TestTab = new TestingViewTab();
    connect(testController, SIGNAL(testSetLoaded(QString)), TestTab, SLOT(testSetLoaded(QString)));
    connect(testController, SIGNAL(animateLoading(int)), TestTab, SLOT(animateLoading(int)));
    connect(TestTab, SIGNAL(statusChanged()), this, SLOT(statusChanged()));
    connect(testController, SIGNAL(changeTestMethod(QString*)), TestTab, SLOT(changeTestMethod(QString*)));
    connect(testController, SIGNAL(doneIteration(double,double,QString*)), TestTab, SLOT(testCycleDone(double,double,QString*)));
    connect(testController, SIGNAL(testStarted(QString*,int,int)), TestTab, SLOT(testStarted(QString*,int,int)));
    connect(testController, SIGNAL(testStopped(QString*)), TestTab, SLOT(testStopped(QString*)));
    connect(testController,SIGNAL(testSetLoaded(NNData*)), TestTab, SIGNAL(testSetLoaded(NNData*)));
    connect(testController, SIGNAL(changeTestMethod(NNTester*)), TestTab, SIGNAL(changeTestMethod(NNTester*)));

    TrainTab = new TrainingViewTab();
    connect(trainerController, SIGNAL(trainSetLoaded(QString)), TrainTab, SLOT(trainSetLoaded(QString)));
    connect(trainerController, SIGNAL(animateLoading(int)), TrainTab, SLOT(animateLoading(int)));
    connect(TrainTab, SIGNAL(statusChanged()), this, SLOT(statusChanged()));
    connect(trainerController, SIGNAL(changeTrainMethod(QString*)), TrainTab, SLOT(changeTrainMethod(QString*)));
    connect(trainerController, SIGNAL(doneIteration(double,double,QString*)), TrainTab, SLOT(trainCycleDone(double,double,QString*)));
    connect(trainerController, SIGNAL(trainStarted(QString*,int,int, int, double, double)), TrainTab, SLOT(trainStarted(QString*,int,int, int, double, double)));
    connect(trainerController, SIGNAL(trainStopped(QString*)), TrainTab, SLOT(trainStopped(QString*)));
    connect(trainerController, SIGNAL(trainStarted(QString*,int,int,int,double,double)), TrainTab, SIGNAL(startTrain(QString*, int, int, int, double, double)));
    connect(trainerController, SIGNAL(changeTrainMethod(NNTrainer*)), TrainTab, SIGNAL(changeTrainMethod(NNTrainer*)));
    connect(trainerController, SIGNAL(constantsChanged(int,int,double,double)), TrainTab, SIGNAL(constantsChanged(int, int, double, double)));
    connect(trainerController, SIGNAL(doneIteration(double,double,QString*)), TrainTab, SIGNAL(doneIteration(double, double, QString*)));

    /* Add the tabs and their icons to the widget */
    QIcon icon1, icon2, icon3, icon4;
    icon1.addFile(QString::fromUtf8(":/new/icons/network.ico"), QSize(), QIcon::Normal, QIcon::Off);
    icon2.addFile(QString::fromUtf8(":/new/icons/Test.ico"),    QSize(), QIcon::Normal, QIcon::Off);
    icon3.addFile(QString::fromUtf8(":/new/icons/Train.ico"),   QSize(), QIcon::Normal, QIcon::Off);
    icon4.addFile(QString::fromUtf8(":/new/icons/TAC.ico"),     QSize(), QIcon::Normal, QIcon::Off);

    TabWidget->addTab(NetworkTab, icon1, QString());
    TabWidget->addTab(DeviceTab,  icon4, QString());
    TabWidget->addTab(TestTab,    icon2, QString());
    TabWidget->addTab(TrainTab,   icon3, QString());

    /* Set the name and tooltips of the tabs */
    TabWidget->setTabText(TabWidget->indexOf(NetworkTab), "Network");
    TabWidget->setTabToolTip(TabWidget->indexOf(NetworkTab), "Allows you to access your neural network");

    TabWidget->setTabText(TabWidget->indexOf(TestTab), "Test");
    TabWidget->setTabToolTip(TabWidget->indexOf(TestTab), "Test the intellegence of your neural network");

    TabWidget->setTabText(TabWidget->indexOf(TrainTab), "Train");
    TabWidget->setTabToolTip(TabWidget->indexOf(TrainTab), "Train your neural network");

    TabWidget->setTabText(TabWidget->indexOf(DeviceTab), "Device");
    TabWidget->setTabToolTip(TabWidget->indexOf(DeviceTab), "Use the neural network with a device");

    connect(TabWidget, SIGNAL(currentChanged(int)), this, SLOT(statusChanged()));

    /* Setup start state of tabs */
    TabWidget->setCurrentIndex(0);

    TabWidget->setTabEnabled(TabWidget->indexOf(TestTab), false);
    TabWidget->setTabEnabled(TabWidget->indexOf(TrainTab), false);
    TabWidget->setTabEnabled(TabWidget->indexOf(DeviceTab), false);

}

void MainWindow::createFileMenu()
{
    /* Create Menu Items */
    newAction = new QAction(tr("&New"), this);
    newAction->setShortcuts(QKeySequence::New);
    newAction->setStatusTip(tr("Create a new neural network"));
    connect(newAction, SIGNAL(triggered()), this, SIGNAL(newFile()));

    openAction = new QAction(tr("&Open"), this);
    openAction->setShortcuts(QKeySequence::Open);
    openAction->setStatusTip(tr("Open a neural network"));
    connect(openAction, SIGNAL(triggered()), this, SIGNAL(open()));

    saveAction = new QAction(tr("&Save"), this);
    saveAction->setShortcuts(QKeySequence::Save);
    saveAction->setStatusTip(tr("Save the neural network"));
    saveAction->setDisabled(true);
    connect(saveAction, SIGNAL(triggered()), this, SIGNAL(save()));

    saveAsAction = new QAction(tr("S&ave As"), this);
    saveAsAction->setShortcuts(QKeySequence::SaveAs);
    saveAsAction->setStatusTip(tr("Save the neural network"));
    saveAsAction->setDisabled(true);
    connect(saveAsAction, SIGNAL(triggered()), this, SIGNAL(saveAs()));

    randomAction = new QAction(tr("&Randomize"), this);
    randomAction->setStatusTip(tr("Randomize the weights of the neural network"));
    randomAction->setDisabled(true);
    connect(randomAction, SIGNAL(triggered()), this, SIGNAL(random()));

    exitAction = new QAction(tr("&Exit"), this);
    exitAction->setShortcut(QKeySequence("Alt+F4"));
    exitAction->setStatusTip(tr("Close the application"));
    connect(exitAction, SIGNAL(triggered()), this, SLOT(exitApp()));

    /* File Menu */
    fileMenu = menuBar()->addMenu(tr("&File"));
    fileMenu->addAction(newAction);
    fileMenu->addAction(openAction);
    fileMenu->addAction(saveAction);
    fileMenu->addAction(saveAsAction);
    fileMenu->addSeparator();
    fileMenu->addAction(randomAction);
    fileMenu->addSeparator();
    fileMenu->addAction(exitAction);
}

void MainWindow::createDeviceMenu()
{
    /* Create Menu Items */
    storeAction = new QAction(tr("&Store to Device"), this);
    storeAction->setStatusTip(tr("Store weights to the device"));
    storeAction->setDisabled(true);
    connect(storeAction, SIGNAL(triggered()), this, SIGNAL(store()));

    loadAction = new QAction(tr("&Load from Device"), this);
    loadAction->setStatusTip(tr("Load weights from the device"));
    loadAction->setDisabled(true);
    connect(loadAction, SIGNAL(triggered()), this, SIGNAL(load()));

    deviceOptionsAction = new QAction(tr("&Device Options"), this);
    deviceOptionsAction->setStatusTip(tr("Configure the external device"));
    deviceOptionsAction->setDisabled(true);
    connect(deviceOptionsAction, SIGNAL(triggered()), this, SIGNAL(deviceOptions()));

    /* Device Menu */
    deviceMenu = menuBar()->addMenu(tr("&Device"));
    deviceMenu->addAction(storeAction);
    deviceMenu->addAction(loadAction);
    deviceMenu->addSeparator();
    deviceMenu->addAction(deviceOptionsAction);
}

void MainWindow::createTestMenu()
{
    /* Create Menu Items */
    loadTestAction = new QAction(tr("&Open Test Set"), this);
    loadTestAction->setStatusTip(tr("Opens a test set to use for testing"));
    loadTestAction->setDisabled(true);
    connect(loadTestAction, SIGNAL(triggered()), this, SIGNAL(loadTest()));

    startTestAction = new QAction(tr("&Start Testing"), this);
    startTestAction->setStatusTip(tr("Starts testing the neural network"));
    startTestAction->setDisabled(true);
    connect(startTestAction, SIGNAL(triggered()), this, SIGNAL(startTest()));

    stopTestAction = new QAction(tr("S&top Testing"), this);
    stopTestAction->setStatusTip(tr("Stops testing the neural network"));
    stopTestAction->setDisabled(true);
    connect(stopTestAction, SIGNAL(triggered()), this, SIGNAL(stopTest()));

    testOptionsAction = new QAction(tr("T&esting Options"), this);
    testOptionsAction->setStatusTip(tr("Options to configure testing"));
    testOptionsAction->setDisabled(true);
    connect(testOptionsAction, SIGNAL(triggered()), this, SIGNAL(testOptions()));

    /* Test Menu */
    testMenu = menuBar()->addMenu(tr("&Test"));
    testMenu->addAction(loadTestAction);
    testMenu->addAction(startTestAction);
    testMenu->addAction(stopTestAction);
    testMenu->addSeparator();
    testMenu->addAction(testOptionsAction);
}

void MainWindow::createTrainMenu()
{
    /* Create Menu Items */
    loadTrainAction = new QAction(tr("&Open Training Set"), this);
    loadTrainAction->setStatusTip(tr("Opens a training set to use for training"));
    loadTrainAction->setDisabled(true);
    connect(loadTrainAction, SIGNAL(triggered()), this, SIGNAL(loadTrain()));

    startTrainAction = new QAction(tr("&Start Training"), this);
    startTrainAction->setStatusTip(tr("Starts training the neural network"));
    startTrainAction->setDisabled(true);
    connect(startTrainAction, SIGNAL(triggered()), this, SIGNAL(startTrain()));

    stopTrainAction = new QAction(tr("S&top Training"), this);
    stopTrainAction->setStatusTip(tr("Stops training the neural network"));
    stopTrainAction->setDisabled(true);
    connect(stopTrainAction, SIGNAL(triggered()), this, SIGNAL(stopTrain()));

    trainOptionsAction = new QAction(tr("T&raining Options"), this);
    trainOptionsAction->setStatusTip(tr("Options to configure training"));
    trainOptionsAction->setDisabled(true);
    connect(trainOptionsAction, SIGNAL(triggered()), this, SIGNAL(trainOptions()));

    /* Train Menu */
    trainMenu = menuBar()->addMenu(tr("&Train"));
    trainMenu->addAction(loadTrainAction);
    trainMenu->addAction(startTrainAction);
    trainMenu->addAction(stopTrainAction);
    trainMenu->addSeparator();
    trainMenu->addAction(trainOptionsAction);
}

void MainWindow::createHelpMenu()
{
    /* Create Menu Items */
    userGuideAction = new QAction(tr("&User Guide"), this);
    userGuideAction->setStatusTip(tr("Opens the user guide"));
    connect(userGuideAction, SIGNAL(triggered()), this, SIGNAL(userGuide()));

    aboutAction = new QAction(tr("&About"), this);
    aboutAction->setStatusTip(tr("Information about TAC"));
    connect(aboutAction, SIGNAL(triggered()), this, SIGNAL(about()));

    /* Help Menu */
    helpMenu = menuBar()->addMenu(tr("&Help"));
    helpMenu->addAction(userGuideAction);
    helpMenu->addAction(aboutAction);
}

void MainWindow::createMenus()
{
    createFileMenu();
    createDeviceMenu();
    createTestMenu();
    createTrainMenu();
    createHelpMenu();
}

void MainWindow::exitApp()
{
    cancelClose = false;

    emit close();

    if(cancelClose == true)
        return;

    exit(0);
}

void MainWindow::closeCancelled()
{
    cancelClose = true;
}

void MainWindow::activateSave()
{
    saveAction->setEnabled(true);
    saveAsAction->setEnabled(true);
}

void MainWindow::deactivateSave()
{
    saveAction->setEnabled(false);
    saveAsAction->setEnabled(false);
}

void MainWindow::openNetwork(NNApprox* nn, QString* filename)
{
    randomAction->setEnabled(true);
}

void MainWindow::createControllers()
{
    /* File Controller */
    fileController = new FileController();

    connect(this, SIGNAL(newFile()), fileController, SLOT(newFile()));
    connect(this, SIGNAL(open()), fileController, SLOT(open()));
    connect(this, SIGNAL(save()), fileController, SLOT(save()));
    connect(this, SIGNAL(saveAs()), fileController, SLOT(saveAs()));
    connect(this, SIGNAL(random()), fileController, SLOT(random()));
    connect(this, SIGNAL(close()), fileController, SLOT(close()));
    connect(this, SIGNAL(networkUpdated(NNApprox*)), fileController, SLOT(updateNetwork(NNApprox*)));

    connect(fileController, SIGNAL(activateSave()), this, SLOT(activateSave()));
    connect(fileController, SIGNAL(deactivateSave()), this, SLOT(deactivateSave()));
    connect(fileController, SIGNAL(closeCancelled()), this, SLOT(closeCancelled()));
    connect(fileController, SIGNAL(changedFilename(QString*)), this, SIGNAL(changedFilename(QString*)));
    connect(fileController, SIGNAL(networkUpdated(NNApprox*)), this, SIGNAL(networkUpdated(NNApprox*)));
    connect(fileController, SIGNAL(networkOpened(NNApprox*,QString*)), this, SIGNAL(networkOpened(NNApprox*,QString*)));

    connect(this, SIGNAL(networkOpened(NNApprox*, QString*)), this, SLOT(openNetwork(NNApprox*, QString*)));

    /* Device Controller */
    deviceController = new DeviceController();

    connect( deviceController, SIGNAL(networkUpdated(NNApprox*)), this, SIGNAL(networkUpdated(NNApprox*)) );
    connect( this, SIGNAL(networkOpened(NNApprox*,QString*)), deviceController, SLOT(openNetwork(NNApprox*,QString*)));
    connect(deviceController, SIGNAL(enableDeviceOptions()), this, SLOT(enableDeviceOptions()));
    connect(deviceController, SIGNAL(enableStoreLoad()), this, SLOT(enableStoreLoad()));
    connect(deviceController, SIGNAL(disableStoreLoad()), this, SLOT(disableStoreLoad()));
    connect(this, SIGNAL(store()), deviceController, SLOT(store()));
    connect(this, SIGNAL(load()), deviceController, SLOT(load()));
    connect(this, SIGNAL(deviceOptions()), deviceController, SLOT(deviceOptions()));
    connect(deviceController, SIGNAL(enableDeviceTab()), this, SLOT(enableDeviceTab()));
    connect(deviceController, SIGNAL(deviceChanged(NNDevice*)), this, SIGNAL(deviceChanged(NNDevice*)));

    /* Test Controller */
    testController = new TestController();

    connect(this, SIGNAL(networkOpened(NNApprox*,QString*)), testController, SLOT(openNetwork(NNApprox*,QString*)));
    connect(testController, SIGNAL(setTestStop(bool)), this, SLOT(setTestStop(bool)));
    connect(testController, SIGNAL(setTestStart(bool)), this, SLOT(setTestStart(bool)));
    connect(testController, SIGNAL(enableTestTab()), this, SLOT(enableTestTab()));
    connect(this, SIGNAL(startTest()), testController, SLOT(startTest()));
    connect(this, SIGNAL(stopTest()), testController, SLOT(stopTest()));
    connect(this, SIGNAL(testOptions()), testController, SLOT(testOptions()));
    connect(this, SIGNAL(loadTest()), testController, SLOT(loadTestSet()));
    connect(testController, SIGNAL(setTestLoad(bool)), this, SLOT(setTestLoad(bool)));

    /* Training Controller */
    trainerController = new TrainerController();

    connect(this, SIGNAL(networkOpened(NNApprox*,QString*)), trainerController, SLOT(openNetwork(NNApprox*,QString*)));
    connect(trainerController, SIGNAL(setTrainStop(bool)), this, SLOT(setTrainStop(bool)));
    connect(trainerController, SIGNAL(setTrainStart(bool)), this, SLOT(setTrainStart(bool)));
    connect(trainerController, SIGNAL(enableTrainTab()), this, SLOT(enableTrainTab()));
    connect(this, SIGNAL(startTrain()), trainerController, SLOT(startTrain()));
    connect(this, SIGNAL(stopTrain()), trainerController, SLOT(stopTrain()));
    connect(this, SIGNAL(trainOptions()), trainerController, SLOT(trainOptions()));
    connect(this, SIGNAL(loadTrain()), trainerController, SLOT(loadTrainSet()));
    connect(trainerController, SIGNAL(setTrainLoad(bool)), this, SLOT(setTrainLoad(bool)));

}

void MainWindow::setTrainLoad(bool value)
{
    loadTrainAction->setEnabled(value);
}

void MainWindow::setTrainStart(bool value)
{
    startTrainAction->setEnabled(value);
}

void MainWindow::setTrainStop(bool value)
{
    stopTrainAction->setEnabled(value);
}

void MainWindow::enableTrainTab()
{
    TabWidget->setTabEnabled(TabWidget->indexOf(TrainTab), true);
    trainOptionsAction->setEnabled(true);
}

void MainWindow::setTestLoad(bool value)
{
    loadTestAction->setEnabled(value);
}

void MainWindow::setTestStart(bool value)
{
    startTestAction->setEnabled(value);
}

void MainWindow::setTestStop(bool value)
{
    stopTestAction->setEnabled(value);
}

void MainWindow::enableTestTab()
{
    TabWidget->setTabEnabled(TabWidget->indexOf(TestTab), true);
    testOptionsAction->setEnabled(true);
}

void MainWindow::enableDeviceTab()
{
    TabWidget->setTabEnabled(TabWidget->indexOf(DeviceTab), true);
}

void MainWindow::enableDeviceOptions()
{
    deviceOptionsAction->setEnabled(true);
}

void MainWindow::enableStoreLoad()
{
    storeAction->setEnabled(true);
    loadAction->setEnabled(true);
}

void MainWindow::disableStoreLoad()
{
    storeAction->setEnabled(false);
    loadAction->setEnabled(false);
}

void MainWindow::removedMessage(int id)
{
    for(int i = 0; i < 5; i++)
    {
        if(statusBarPos[i] == id)
        {
            for(int j = i + 1; j < 5; j++)
            {
                statusBarPos[j-1] = statusBarPos[j];
            }
            statusBarPos[4] = -1;

            break;
        }
    }
}

void MainWindow::addedMessage(int id, QWidget* label)
{
    for(int i = 0; i < 5; i++)
    {
        if((statusBarPos[i] > id) ||(statusBarPos[i] == -1))
        {
            for(int j = 4; j > i; j--)
            {
                statusBarPos[j] = statusBarPos[j-1];
            }

            statusBarPos[i] = id;

            if( id > 0 )
                statusBar()->insertPermanentWidget(i, label, ((id == 1) ? 1:0));
            else
                statusBar()->insertWidget(i, label, 0);

            return;
        }
    }

}

void MainWindow::statusChanged()
{

    status* msg = ((TabView*)(TabWidget->currentWidget()))->getStatus();

    statusBar()->setMinimumHeight(20);

    if(msg->tempLabel == NULL)
    {
        statusBar()->removeWidget(tempLabel);

        removedMessage(0);
    }
    else
    {
        tempLabel->setText(msg->tempLabel);

        if(tempLabel->isHidden())
        {
            tempLabel->show();
            addedMessage(0, tempLabel);
        }

        free(msg->tempLabel);
    }

    if(msg->hintLabel == NULL)
    {
        statusBar()->removeWidget(hintLabel);
        removedMessage(1);
    }
    else
    {
        hintLabel->setText(msg->hintLabel);

        if( hintLabel->isHidden() )
        {
            hintLabel->show();
            hintLabel->setAlignment(Qt::AlignCenter);

            addedMessage(1, hintLabel);
        }

        free(msg->hintLabel);
    }

    if(msg->statusLabel == NULL)
    {
        statusBar()->removeWidget(statusLabel);
        removedMessage(2);
    }
    else
    {
        statusLabel->setText(msg->statusLabel);

        if(statusLabel->isHidden())
        {
            statusLabel->show();
            addedMessage(2, statusLabel);
        }

        free(msg->statusLabel);
    }

    if(msg->progressLabel == NULL)
    {
        statusBar()->removeWidget(progressLabel);
        removedMessage(3);
    }
    else
    {
        progressLabel->setText(msg->progressLabel);

        if(progressLabel->isHidden())
        {
            progressLabel->show();
            addedMessage(3, progressLabel);
        }

        free(msg->progressLabel);
    }

    if(msg->progressBar < 0)
    {
        statusBar()->removeWidget(progressBar);
        progressBar->hide();
        removedMessage(4);
    }
    else
    {
        progressBar->setValue(100*msg->progressBar);

        if(progressBar->isHidden())
        {
            progressBar->show();
            addedMessage(4, progressBar);
        }
    }

    free(msg);
}
