/**
* Matthew Netsch
* Feb 2010
*
* MainWindow.h
*
* Contains all of the tabs
* for the program. This also
* acts as a relay for all of the
* messages that need to be
* passed between group boxes.
*/

#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QtCore/QVariant>
#include <QtGui/QAction>
#include <QtGui/QApplication>
#include <QtGui/QButtonGroup>
#include <QtGui/QGraphicsView>
#include <QtGui/QGridLayout>
#include <QtGui/QGroupBox>
#include <QtGui/QHBoxLayout>
#include <QtGui/QHeaderView>
#include <QtGui/QLabel>
#include <QtGui/QMainWindow>
#include <QtGui/QProgressBar>
#include <QtGui/QPushButton>
#include <QtGui/QTabWidget>
#include <QtGui/QVBoxLayout>
#include <QtGui/QWidget>
#include <QtGui/QTableWidget>
#include <QtGui/QComboBox>
#include <QtGui/QFileDialog>
#include <QtGui/QMessageBox>
#include <QtGui/QMenu>
#include <QtGui/QMenuBar>
#include <QtGui/QStatusBar>
#include <QtGui/QSplitter>

#include "TAC/Control/FileController.h"
#include "TAC/Control/DeviceController.h"
#include "TAC/Control/TestController.h"
#include "TAC/Control/TrainerController.h"
#include "TAC/View/NetworkViewTab.h"
#include "TAC/View/DeviceViewTab.h"
#include "TAC/View/TestingViewTab.h"
#include "TAC/view/TrainingViewTab.h"

QT_BEGIN_NAMESPACE
class QWidget;
class QHBoxLayout;
class QTabWidget;
class QGridLayout;
class QGroupBox;
class QLabel;
class QPushButtons;
class QGraphicsView;
class QVBoxLayout;
class QProgressBar;
class QTableWidget;
class QComboBox;
class QFileDialog;
class QMessageBox;
QT_END_NAMESPACE

class MainWindow : public QMainWindow
{
   Q_OBJECT

public:
    MainWindow();

signals:
    void newFile();
    void open();
    void save();
    void saveAs();
    void random();
    void close();

    void store();
    void load();
    void deviceOptions();

    void loadTest();
    void startTest();
    void stopTest();
    void testOptions();

    void loadTrain();
    void startTrain();
    void stopTrain();
    void trainOptions();

    void userGuide();
    void about();

    void networkUpdated(NNApprox* nn);
    void networkOpened(NNApprox* nn, QString* filename);
    void changedFilename(QString* filename);

    void deviceChanged(NNDevice* device);

public slots:
    void exitApp();

    void closeCancelled();

    void activateSave();
    void deactivateSave();

    void openNetwork(NNApprox* nn, QString* filename);

    void statusChanged();

    void enableStoreLoad();
    void disableStoreLoad();
    void enableDeviceOptions();

    void setTestStart(bool value);
    void setTestStop(bool value);
    void setTestLoad(bool value);

    void setTrainStart(bool value);
    void setTrainStop(bool value);
    void setTrainLoad(bool value);

    void enableDeviceTab();
    void enableTestTab();
    void enableTrainTab();

private:

    bool cancelClose;

    /* Controllers */
    FileController* fileController;
    DeviceController* deviceController;
    TestController* testController;
    TrainerController* trainerController;

    void createControllers();
    void addedMessage(int id, QWidget* label);
    void removedMessage(int id);

    QLabel* tempLabel;
    QLabel* hintLabel;
    QLabel* statusLabel;
    QLabel* progressLabel;
    QProgressBar* progressBar;

    int statusBarPos[5];

    /* Constructing */
    void createTabs();
    void createFileMenu();
    void createDeviceMenu();
    void createTestMenu();
    void createTrainMenu();
    void createHelpMenu();
    void createMenus();

    /* Widgets */
    QTabWidget            *TabWidget;
    NetworkViewTab        *NetworkTab;
    TestingViewTab        *TestTab;
    TrainingViewTab       *TrainTab;
    DeviceViewTab         *DeviceTab;

    /* Menus */
    QMenu* fileMenu;
    QMenu* deviceMenu;
    QMenu* testMenu;
    QMenu* trainMenu;
    QMenu* helpMenu;

    /* Menu Items */
    QAction* newAction;
    QAction* openAction;
    QAction* saveAction;
    QAction* saveAsAction;
    QAction* randomAction;
    QAction* exitAction;

    QAction* storeAction;
    QAction* loadAction;
    QAction* deviceOptionsAction;

    QAction* loadTestAction;
    QAction* startTestAction;
    QAction* stopTestAction;
    QAction* testOptionsAction;

    QAction* loadTrainAction;
    QAction* startTrainAction;
    QAction* stopTrainAction;
    QAction* trainOptionsAction;

    QAction* userGuideAction;
    QAction* aboutAction;

};

#endif // MAINWINDOW_H
