#include "DeviceController.h"

DeviceController::DeviceController()
{

    currentDevice  = NULL;

    myNetwork = NULL;

}

/* SLOT that gets called when a new network is opened */
void DeviceController::openNetwork(NNApprox* nn, QString* filename)
{
    myNetwork = nn;

    if((currentDevice != NULL)&&(currentDevice->getVersion() != myNetwork->getVersion()))
    {
        changeDevice(NULL);
    }

    emit enableDeviceOptions();
    emit enableDeviceTab();
}

void DeviceController::store()
{
    /* Get the current device and call it's store option */
    if(currentDevice == NULL)
        return;

    currentDevice->store(myNetwork);
}

void DeviceController::load()
{
    if(currentDevice == NULL)
        return;

    currentDevice->load(myNetwork);

    emit networkUpdated(myNetwork);
}

void DeviceController::deviceOptions()
{
    if( myNetwork == NULL )
    {
        error("Cannot display options", "No neural network loaded");
        return;
    }

    DeviceProviderLocator* locator = ServiceLocator::locator(myNetwork->getVersion());

    if(locator == NULL)
    {
        error("Cannot display options", "Cannot find device provider locator");
        return;
    }

    DeviceProvider* provider = locator->LocateDeviceProvider();

    if(provider == NULL)
    {
         error("Cannot display options", "Cannot find device provider");
         return;
    }

    if(provider->isRunning() == false)
    {
        provider->start();
        while(provider->isRunning() == false);
        connect(provider, SIGNAL(updateDevices()), this, SLOT(updateDevices()));
    }

    DeviceOptionsDialog* dialog = new DeviceOptionsDialog(provider);

    connect(dialog, SIGNAL(changeDevice(NNDevice*)), this, SLOT(changeDevice(NNDevice*)));

    if( dialog->isReady() == false )
    {
        error("Cannot display options", "No devices available");
        return;
    }

    dialog->exec();

    delete(dialog);
}

void DeviceController::updateDevices()
{
    if(currentDevice == NULL)
        return;

    if( myNetwork == NULL )
    {
        error("Lost device connection", "Internal neural network corrupted");
        changeDevice(NULL);
        return;
    }

    DeviceProviderLocator* locator = ServiceLocator::locator(myNetwork->getVersion());

    if(locator == NULL)
    {
        error("Lost device connection", "No devices available");
        changeDevice(NULL);
        return;
    }

    DeviceProvider* provider = locator->LocateDeviceProvider();

    if(provider == NULL)
    {
         error("Lost device connection", "No devices available");
         changeDevice(NULL);
         return;
    }

    int len = 0;
    NNDevice** devices = provider->queryDevices(&len);

    for(int i = 0; i < len; i++)
    {
        if(currentDevice == devices[i])
        {
            return;
        }
    }

    changeDevice(NULL);

    error("Lost device connection", "Device not connected");
}

void DeviceController::changeDevice(NNDevice* currentDevice)
{
   this->currentDevice = currentDevice;

   if(currentDevice == NULL)
   {
       emit disableStoreLoad();
   }
   else
   {
       emit enableStoreLoad();
   }

   emit deviceChanged(currentDevice);
}

void DeviceController::error(const char* title, const char* message)
{
    /* Display error message*/
    QIcon icon;
    icon.addFile(QString::fromUtf8(":/new/icons/TAC.ico"), QSize(), QIcon::Normal, QIcon::Off);

    QMessageBox msg;

    msg.setText(message);
    msg.setWindowTitle(title);
    msg.setIcon(QMessageBox::Critical);
    msg.setWindowIcon(icon);
    msg.exec();

}

