#include "DeviceOptionsDialog.h"

DeviceOptionsDialog::DeviceOptionsDialog(DeviceProvider* provider)
{
    ready = true;

    layout   = new QGridLayout();

    label    = new QLabel(tr("Select Device"));

    combo    = new QComboBox();
    combo->setMinimumWidth(120);

    submitButton = new QPushButton();
    submitButton->setMaximumWidth(80);
    submitButton->setMinimumWidth(80);
    submitButton->setText("OK");

    cancelButton = new QPushButton();
    cancelButton->setMaximumWidth(80);
    cancelButton->setMinimumWidth(80);
    cancelButton->setText("Cancel");

    configureButton = new QPushButton();
    configureButton->setMaximumWidth(80);
    configureButton->setMinimumWidth(80);
    configureButton->setText("Configure");
    configureButton->setEnabled(false);

    connect(submitButton, SIGNAL(clicked()), this, SLOT(submitted()));
    connect(cancelButton, SIGNAL(clicked()), this, SLOT(cancelled()));
    connect(configureButton, SIGNAL(clicked()), this, SLOT(configure()));

    setWindowTitle(tr("Device Options"));

    QIcon icon;
    icon.addFile(QString::fromUtf8(":/new/icons/TAC.ico"), QSize(), QIcon::Normal, QIcon::Off);
    setWindowIcon(icon);

    setFixedSize( 300, 100 );
    setAutoFillBackground(true);

    int len = 0;
    NNDevice** devices = provider->queryDevices(&len);

    if(len <= 0 )
    {
        ready = false;
        return;
    }

    /* Populate Combo */
    combo->addItem(tr("None"), ((int)NULL));

    for(int i = 0; i < len; i++)
    {
       int addr = (int)devices[i];
       combo->addItem(*(devices[i]->getName()), addr);
    }

    connect(combo, SIGNAL(currentIndexChanged(int)), this, SLOT(indexChanged(int)));

    currentDevice = NULL;

    layout->addWidget(label, 0, 0, 1, 1, Qt::AlignLeft);
    layout->addWidget(combo, 0, 1, 1, 1, Qt::AlignLeft);
    layout->addWidget(configureButton, 0, 2, 1, 1, Qt::AlignRight);
    layout->addWidget(submitButton, 2, 1, 1, 1, Qt::AlignRight);
    layout->addWidget(cancelButton, 2, 2, 1, 1, Qt::AlignRight);

    setLayout(layout);
}

bool DeviceOptionsDialog::isReady()
{
    return ready;
}

void DeviceOptionsDialog::cancelled()
{
    close();
}

void DeviceOptionsDialog::submitted()
{
    emit changeDevice(currentDevice);
    close();
}

void DeviceOptionsDialog::configure()
{
   if(currentDevice != NULL)
       currentDevice->configure();
}

void DeviceOptionsDialog::indexChanged(int i)
{
    if( i == 0 )
    {
        currentDevice = NULL;
        configureButton->setEnabled(false);
        return;
    }

    bool ok = 0;
    currentDevice = (NNDevice*)(combo->itemData(i).toInt(&ok));

    if(currentDevice == NULL)
    {
        configureButton->setEnabled(false);
        return;
    }

    if(currentDevice->isConfigurable() == true)
    {
        configureButton->setEnabled(true);
    }
    else
    {
        configureButton->setEnabled(false);
    }
}
