/**
* Matthew Netsch
* Feb 2010
*
* NewFileDialog.cpp
*
* Dialog to select new file.
*/

#include "NewFileDialog.h"

/* Constructor */
NewFileDialog::NewFileDialog()
{
    /* Setup Widgets */
    layout   = new QGridLayout();

    label    = new QLabel(tr("Please select the type of neural network"));

    combo    = new QComboBox();
    combo->setMinimumWidth(120);

    int len = 0;
    ServiceLocator** locators = ServiceLocator::getInstances(&len);

    if( len != 0 )
    {
       for(int i = 0; i < len; i++)
       {
         char* name = locators[i]->toString();

         combo->addItem(QString(name), QVariant(locators[i]->getVersion()));

         if(name != NULL)
          free(name);
       } 

       ready = true;

    } 
    else
    {
      ready = false;
    }

    submitButton = new QPushButton();
    submitButton->setMaximumWidth(80);
    submitButton->setMinimumWidth(80);
    submitButton->setText("OK");

    cancelButton = new QPushButton();
    cancelButton->setMaximumWidth(80);
    cancelButton->setMinimumWidth(80);
    cancelButton->setText("Cancel");

    connect(submitButton, SIGNAL(clicked()), this, SLOT(submitted()));
    connect(cancelButton, SIGNAL(clicked()), this, SLOT(cancelled()));

    setWindowTitle(tr("New Neural Network"));

    QIcon icon;
    icon.addFile(QString::fromUtf8(":/new/icons/TAC.ico"), QSize(), QIcon::Normal, QIcon::Off);
    setWindowIcon(icon);

    setFixedSize( 250, 100 );
    setAutoFillBackground(true);

    layout->addWidget(label, 0, 0, 1, 3, Qt::AlignLeft);
    layout->addWidget(combo, 1, 0, 1, 3, Qt::AlignLeft);
    layout->addWidget(submitButton, 3, 1, 1, 1, Qt::AlignRight);
    layout->addWidget(cancelButton, 3, 2, 1, 1, Qt::AlignRight);

    setLayout(layout);

}

void NewFileDialog::cancelled()
{
    close();
}

/* SLOT that gets called when the OK button is pressed */
void NewFileDialog::submitted()
{
    bool ok = true;

    int version = combo->itemData(combo->currentIndex()).toInt(&ok);

    emit submit( (ok) ? version : -1 );

    close();
}

bool NewFileDialog::isReady()
{
    return ready;
}

