#include "TestController.h"

TestController::TestController()
{
    myNetwork = NULL;
    myData    = new NNData();
    tester    = NULL;
    iter      = 0;

    connect(myData, SIGNAL(doneLoading(QString)), this, SLOT(doneLoading(QString)));
    connect(myData, SIGNAL(wrongVersion(int)), this, SLOT(wrongVersion(int)));
    connect(myData, SIGNAL(loading()), this, SLOT(loading()));

    isLoading = false;

    timer.setSingleShot(false);
    connect(&timer, SIGNAL(timeout()), this, SLOT(animate()));
    timer.start(1000);

}

void TestController::openNetwork(NNApprox* nn, QString* filename)
{

    if((myNetwork == NULL)||((nn->getVersion() != myNetwork->getVersion())))
    { /* We have a previously loaded set that doesnt match new network
         -OR-
         We have no neural network loaded
         -THEN-
         (re)load everything
      */
        if((myNetwork != NULL)&&(((nn->getVersion() != myNetwork->getVersion()))))
        {
            myData->destroyData();

            QString result1 = NULL;
            NNData* result2 = NULL;
            emit testSetLoaded(result1);
            emit testSetLoaded(result2);
        }

        emit setTestStart(false);
        emit setTestStop(false);

        /* Stop previous tester */
        if((tester != NULL)&&(tester->isRunning()))
        {
            emit stopTesting();
            while(tester->isRunning());
        }

        TestProviderLocator* locator = ServiceLocator::locator(nn->getVersion());
        TestProvider* provider = NULL;
        NNTester** testers = NULL;
        int len = 0;
        tester = NULL;

        if(locator != NULL)
            provider = locator->LocateTestProvider();

        if(provider != NULL)
            testers = provider->queryTesters(&len);

        if( len > 0 )
            changeTester(testers[0]);
        else
        {
            NNTester* result1 = NULL;
            QString* result2 = NULL;
            emit changeTestMethod(result1);
            emit changeTestMethod(result2);
        }

        if((tester != NULL)&&(myData->getNumData() > 0))
            emit setTestStart(true);

        iter = 0;

    }

    myNetwork = nn;
    myData->setVersion(myNetwork->getVersion());

    emit enableTestTab();
    emit setTestLoad(true);
}

void TestController::loadTestSet()
{
    /* Open the choose file dialog */
    QString filename = QFileDialog::getOpenFileName(0,
                                                   QString(),
                                                   QString(),
                                                   "TAC Test Files (*.tac)");

    /* If user did not make a choice then return */
    if( filename == NULL )
        return;

    /* Disable the load button and load the file */
    emit setTestLoad(false);

    myData->run(filename);

}

void TestController::doneLoading(QString filename)
{

    isLoading = false;
    emit setTestLoad(true);

    if(tester != NULL)
        emit setTestStart(true);

    emit testSetLoaded(filename);
    emit testSetLoaded(myData);
}

void TestController::wrongVersion(int version)
{
    error("Cannot Load File", "File version is not compatible with the neural network");
    setTestLoad(true);
}

void TestController::loading()
{
   count = 0;
   isLoading = true;
}

void TestController::animate()
{
    /* If we are loading then increment display */
    if(isLoading)
    {
        count = (count + 1)%3;
        emit animateLoading(count);
    }
    else
    {
        count = -1;
    }
}

void TestController::startTest()
{
   emit setTestStart(false);
   emit setTestLoad(false);

   emit testStarted(tester->getName(), myData->getNumData(), iter);

   tester->run(myNetwork, myData);

   emit setTestStop(true);
}

void TestController::stopTest()
{
    emit setTestStop(false);

    if(tester->isRunning())
    {
        emit stopTesting();
        while(tester->isRunning());
    }

    emit testStopped(tester->getStats());

    emit setTestStart(true);
    emit setTestLoad(true);
}

void TestController::testOptions()
{
    TestProviderLocator* locator = ServiceLocator::locator(myNetwork->getVersion());

    if(locator == NULL)
    {
        error("Cannot View Options", "No test methods loaded");
        return;
    }

    TestProvider* testProvider = locator->LocateTestProvider();

    if(testProvider == NULL)
    {
        error("Cannot View Options", "No test methods loaded");
        return;
    }

    TestOptionsDialog* testOptions = new TestOptionsDialog(testProvider, tester, iter);

    if(testOptions->isReady() == false)
    {
        error("Cannot View Options", "No test methods loaded");
        delete(testOptions);
        return;
    }

    connect(testOptions, SIGNAL(changeIterEdit(int)), this, SLOT(changeIter(int)));
    connect(testOptions, SIGNAL(changeTester(NNTester*)), this, SLOT(changeTester(NNTester*)));

    testOptions->show();
}

void TestController::changeTester(NNTester* tester)
{
    if(this->tester != NULL)
    { /* Disconnect */
        this->tester->disconnect();
        disconnect(this->tester);
    }

    this->tester = tester;

    if(tester != NULL)
    {
        connect(this, SIGNAL(stopTesting()), tester, SLOT(stopTesting()));
        connect(tester, SIGNAL(doneTesting()), this, SLOT(stopTest()));
        connect(tester, SIGNAL(testCycleDone(double,double, QString*)), this, SLOT(testCycleDone(double,double, QString*)));
        emit setTestStart(false);
        emit setTestStop(false);

        if(myData->getNumData() > 0)
            emit setTestStart(true);

        emit changeTestMethod(tester->getName());
        emit changeTestMethod(tester);
    }
    else
    {
        NNTester* result1 = NULL;
        QString* result2 = NULL;
        emit changeTestMethod(result1);
        emit changeTestMethod(result2);

    }
}

void TestController::testCycleDone(double percentDone, double error, QString* stats)
{
    emit doneIteration(percentDone, error, stats);
}

void TestController::changeIter(int iter)
{
    this->iter = iter;
}

void TestController::error(const char* title, const char* message)
{
    /* Display error message*/
    QIcon icon;
    icon.addFile(QString::fromUtf8(":/new/icons/TAC.ico"), QSize(), QIcon::Normal, QIcon::Off);

    QMessageBox msg;

    msg.setText(message);
    msg.setWindowTitle(title);
    msg.setIcon(QMessageBox::Critical);
    msg.setWindowIcon(icon);
    msg.exec();
}
