#ifndef TESTCONTROLLER_H
#define TESTCONTROLLER_H

#include <QThread>
#include <QMessageBox>
#include <QTimer>
#include <QFileDialog>
#include <QIcon>

#include "../NeuralNetwork/NNApprox.h"
#include "../FileIO/NNData.h"
#include "../ServiceLocator/ServiceLocator.h"
#include "../NeuralNetwork/NNTester.h"
#include "TestOptionsDialog.h"

class TestController : public QThread
{
    Q_OBJECT

public slots:
    void openNetwork(NNApprox* nn, QString* filename);

    void loadTestSet();
    void startTest();
    void stopTest();
    void testOptions();

    void doneLoading(QString);
    void wrongVersion(int);
    void loading();

    void animate();

    void changeTester(NNTester* tester);
    void changeIter(int iter);

    void testCycleDone(double percentDone, double error, QString* stats);

signals:
    void setTestStart(bool value);
    void setTestStop(bool value);
    void setTestLoad(bool value);

    void testStarted(QString* methodName, int numTests, int iter);
    void testStopped(QString* stats);

    void changeTestMethod(QString* methodName);
    void changeTestMethod(NNTester* tester);
    void doneIteration(double percentDone, double error, QString* stats);

    void enableTestTab();

    void animateLoading(int count);
    void testSetLoaded(QString filename);
    void testSetLoaded(NNData* testSet);

    void stopTesting();

private:

   NNApprox* myNetwork;
   NNData*   myData;
   int       iter;
   NNTester* tester;

   bool      isLoading;

   int       count;
   QTimer    timer;

   void error(const char* title, const char* message);

public:
    TestController();
};

#endif // TESTCONTROLLER_H
