#include "TestOptionsDialog.h"

TestOptionsDialog::TestOptionsDialog(TestProvider* provider, NNTester* currentTester, int iter)
{

    this->currentTester = currentTester;
    this->iter          = iter;

    ready = false;

    layout = new QGridLayout();

    testerLabel = new QLabel(tr("Error Determination "));
    iterLabel = new QLabel(tr("Output Every Iteration "));

    char buf[64];
    sprintf(&(buf[0]), "%d", iter);

    iterEdit = new QLineEdit();
    iterEdit->setText(&(buf[0]));
    iterEdit->setValidator(new QIntValidator(0, 1000000, iterEdit));

    combo    = new QComboBox();
    combo->setMinimumWidth(120);

    submitButton = new QPushButton();
    submitButton->setMaximumWidth(80);
    submitButton->setMinimumWidth(80);
    submitButton->setText("OK");

    cancelButton = new QPushButton();
    cancelButton->setMaximumWidth(80);
    cancelButton->setMinimumWidth(80);
    cancelButton->setText("Cancel");

    connect(cancelButton, SIGNAL(clicked()), this, SLOT(cancelled()));
    connect(submitButton, SIGNAL(clicked()), this, SLOT(submitted()));
    connect(iterEdit, SIGNAL(textEdited(QString)), this, SLOT(iterEditted(QString)));

    setWindowTitle(tr("Testing Options"));

    QIcon icon;
    icon.addFile(QString::fromUtf8(":/new/icons/TAC.ico"), QSize(), QIcon::Normal, QIcon::Off);
    setWindowIcon(icon);

    setFixedSize( 315, 120 );
    setAutoFillBackground(true);

    int len = 0;
    NNTester** testers = provider->queryTesters(&len);

    if(len <= 0 )
    {
        ready = false;
        return;
    }

    /* Populate Combo */
    int currentIndex = -1;

    for(int i = 0; i < len; i++)
    {
       int addr = (int)testers[i];
       combo->addItem(*(testers[i]->getName()), addr);

       if(testers[i] == currentTester)
           currentIndex = i;
    }

    if((currentIndex == -1) && (currentTester != NULL))
    {
        ready = false;
        return;
    }

    if(currentIndex != -1)
    {
        combo->setCurrentIndex(currentIndex);
    }
    else
    {
        combo->setCurrentIndex(0);
        currentTester = testers[0];
    }

    connect(combo, SIGNAL(currentIndexChanged(int)), this, SLOT(indexChanged(int)));

    layout->addWidget(testerLabel, 0, 0, 1, 1, Qt::AlignLeft);
    layout->addWidget(combo, 0, 1, 1, 2, Qt::AlignLeft);
    layout->addWidget(iterLabel, 1, 0, 1, 1, Qt::AlignLeft);
    layout->addWidget(iterEdit, 1, 1, 1, 2, Qt::AlignLeft);
    layout->addWidget(submitButton, 2, 1, 1, 1, Qt::AlignRight);
    layout->addWidget(cancelButton, 2, 2, 1, 1, Qt::AlignRight);

    setLayout(layout);

    ready = true;

}

bool TestOptionsDialog::isReady()
{
    return ready;
}

void TestOptionsDialog::cancelled()
{
    close();
}

void TestOptionsDialog::submitted()
{
    emit changeTester(currentTester);
    emit changeIterEdit(iter);
    close();
}


void TestOptionsDialog::iterEditted(QString text)
{
    /* Determine of the inputs are acceptable */
    if(iterEdit->hasAcceptableInput() != true )
    {
        submitButton->setEnabled(false);
    }
    else
    {
        submitButton->setEnabled(true);
    }

    bool ok = false;
    iter = text.toInt(&ok, 10);

}

void TestOptionsDialog::indexChanged(int i)
{

    bool ok = 0;
    currentTester = (NNTester*)(combo->itemData(i).toInt(&ok));

}
