#ifndef TRAINERCONTROLLER_H
#define TRAINERCONTROLLER_H

#include <QThread>
#include <QMessageBox>
#include <QTimer>
#include <QFileDialog>
#include <QIcon>

#include "../NeuralNetwork/NNApprox.h"
#include "../FileIO/NNData.h"
#include "../ServiceLocator/ServiceLocator.h"
#include "../NeuralNetwork/NNTrainer.h"
#include "TrainerOptionsDialog.h"

class TrainerController : public QThread
{
    Q_OBJECT

public slots:
    void openNetwork(NNApprox* nn, QString* filename);

    void loadTrainSet();
    void startTrain();
    void stopTrain();
    void trainOptions();

    void doneLoading(QString);
    void wrongVersion(int);
    void loading();

    void animate();

    void changeTrainer(NNTrainer* trainer);
    void changeConstants(int iter, int maxIterations, double gamma, double momentum);

    void trainCycleDone(double percentDone, double error, QString* stats);

signals:
    void setTrainStart(bool value);
    void setTrainStop(bool value);
    void setTrainLoad(bool value);

    void trainStarted(QString* methodName, int numTrains, int iter, int maxIterations, double gamma, double momentum);
    void constantsChanged(int iter, int maxIterations, double gamma, double momentum);
    void trainStopped(QString* stats);

    void changeTrainMethod(QString* methodName);
    void changeTrainMethod(NNTrainer* trainer);
    void doneIteration(double percentDone, double error, QString* stats);

    void enableTrainTab();

    void animateLoading(int count);
    void trainSetLoaded(QString filename);

    void stopTraining();

private:

   NNApprox* myNetwork;
   NNData*   myData;

   int        maxIterations;
   int       iter;
   double     gamma;
   double     momentum;
   NNTrainer* trainer;

   bool      isLoading;

   int       count;
   QTimer    timer;

   void error(const char* title, const char* message);

public:
    TrainerController();
};

#endif // TRAINERCONTROLLER_H
