/**
* Matthew Netsch
* Feb 2010
*
* NNReader.cpp
*
* Reader for reading a neural network file.
* Reads the version, constructs the factory
* that matches the version type, and then
* instructs it to create the network.
*/

#include "NNReader.h"

/* Reads the version from the file and constructs the appropiate factory
 * to build the network */
NNApprox* NNReader::read( FILE* pFile )
{
          
          /* Read the version of neural network */
          __int32 version = 0;
          fread( &version, sizeof(__int32), 1, pFile );
          

          /* Read the number of weights */
          __int32 numWeights = 0;
          fread( &numWeights, sizeof(__int32), 1, pFile );
          
          /* Reads the weights */
          double* weights = (double*)malloc( sizeof(double) * numWeights );
          fread( &(weights[0]), sizeof(double), numWeights, pFile);

          /* Attempt to create the neural network */
	  NNFactoryLocator* locator = ServiceLocator::locator(version);

          NNApprox* result = (locator == NULL) ? NULL : locator->LocateNNFactory()->build(weights, (int)numWeights);

          free(weights);
          
          return result;
          
}
