#ifndef __NN__
#define __NN__

#include <cstdlib>
#include <cstring>
#include <cmath>
#include <cstdio>
#include <ctime>

class nn
{
private:
  double** nodes;
  double** error_signals;
  
  int      numLayers;
  int*     node_counts;
  double*  outValues;
  
  double** weights;
  double** weight_deltas;
  
  double   gamma;
  double   momentum;
  
  void destroyNet();
  void constructNet(int* node_counts, int len, double gamma, double momentum);

public:
  
    nn(int* node_counts, int len, double gamma, double momentum);
    ~nn();
    
    double* feed_forward(double* inputs, int inLength, int* outLength);
    double back_propagate(double* expected_outputs, int len);
    
    void store_weights(double* weights, int len);
    double* load_weights(int* len);
    
    void setGamma(double gamma);
    void setMomentum(double momentum);
    void randomize();
    void clone(nn* net);

    int* getCounts(int* numLayers);
    double* getOutputWeights( int layerNum, int nodeNum, int* numOutputs, int* receiverIndex);
    void setWeight(int layer, int node, int receiverNode, double value);
      
};

#endif
