/**
* Matthew Netsch
* Feb 2010
*
* NNApprox.h
*
* Abstract class that contains the methods
* required to train and fire a neural network.
* Also contains the structure of a neural network
* constructed by a subclass of this.
*/

#include "NNApprox.h"

/* Destructor */
NNApprox::~NNApprox()
{
 if( net != NULL )
  delete(net);
}

/* Initiate a training cycle with the given inputs and output */
double NNApprox::train( double* expectedValues, int expLen )
{
       return net->back_propagate(expectedValues, expLen);
}

/* Initiate a forward propagate cycle with the given inputs */
double* NNApprox::fire( double* positions, int len, int* outLen )
{
       return net->feed_forward(positions, len, outLen);
}

/* Get the version this network implements */
int NNApprox::getVersion()
{
  return version;   
}

/* Randomize the weights of the network */
void NNApprox::randomizeWeights()
{
    net->randomize();
}

/* Sets the step increment for all of the weights during back propagation */
void NNApprox::setGamma(double gamma)
{
   net->setGamma(gamma);
}

void NNApprox::setMomentum(double momentum)
{
    net->setMomentum(momentum);
}

/* Deep copy the object given */
void NNApprox::clone(NNApprox* approx)
{
    net->clone(approx->net);
}

int* NNApprox::getCounts(int* numLayers)
{
    return net->getCounts(numLayers);
}

double* NNApprox::getOutputWeights( int layerNum, int nodeNum, int* numOutputs, int* receiverIndex)
{
    return net->getOutputWeights( layerNum, nodeNum, numOutputs, receiverIndex);
}

void NNApprox::setWeight(int layer, int node, int receiverNode, double value)
{
    net->setWeight(layer, node, receiverNode, value);
}

void NNApprox::store_weights(double* weights, int len)
{
    net->store_weights(weights, len);
}

double* NNApprox::load_weights(int* len)
{
    return net->load_weights(len);
}
