/**
* Matthew Netsch
* Feb 2010
*
* NNApprox.h
*
* Abstract class that contains the methods
* required to train and fire a neural network.
* Also contains the structure of a neural network
* constructed by a subclass of this.
*/

#ifndef __NNApprox__
#define __NNApprox__

#include "NN.h"

class NNApprox
{
    protected:  

           /*Neural Network */
           nn*           net;

           int           version;

           /*Training*/ 
           virtual void build() = 0;
      
    public:
           virtual ~NNApprox();
           
           /*Train and Fire*/
           double  train( double* expectedValue, int expLen );
           double* fire( double* positions, int len, int* outLen );
           
           /* Mutators */
           void randomizeWeights();
           void clone(NNApprox* approx);
           void setGamma(double gamma);
           void setMomentum(double momentum);

           void store_weights(double* weights, int len);
           double* load_weights(int* len);

           int* getCounts(int* numLayers);
           double* getOutputWeights( int layerNum, int nodeNum, int* numOutputs, int* receiverIndex);

           void setWeight(int layer, int node, int receiverNode, double value);

           int getVersion();
};

#endif
