/**
* Matthew Netsch
* Feb 2010
*
* NNDevice.h
*
* Interface for a neural network device.
* A device is something that allows to be
* queried for its weights, inputs, outputs,
* and version. It also allows to have its
* weights and inputs changed. Finally you
* can initiate a forward propagation the the
* device. You may also configure a device.
*/

#ifndef NNDEVICE_H
#define NNDEVICE_H

#include "../NeuralNetwork/NNApprox.h"
#include "../NeuralNetwork/NNDevice.h"
#include "../View/DeviceView.h"
#include <QString>
#include <QWidget>

class NNDevice
{
public:
    /* Version and Name */
    virtual int getVersion() = 0;
    virtual QString* getName() = 0;

    /* Weights */
    virtual void load(NNApprox* approx) = 0;
    virtual void store(NNApprox* approx) = 0;

    /* Connection */
    virtual bool isConnected() = 0;
    virtual void testConnection() = 0;
    virtual void setConnected(bool connected) = 0;

    /* Configure */
    virtual void configure() = 0;
    virtual bool isConfigurable() = 0;

    virtual void reload() = 0 ;

    /* Evaluate */
    virtual double evaluate(double* inputs, int numInputs) = 0;

    /* View */
    virtual void constructViews(NNDevice* device, DeviceView** view, QWidget** controls) = 0;

};

#endif // NNDEVICE_H
