/**
* Matthew Netsch
* Feb 2010
*
* TesterThread.cpp
*
* A thread that performs testing on
* a neural network and publishes the
* results.
*/

#include "NNTester.h"

/* Constructor */
NNTester::NNTester()
{
    myNetwork   = NULL;
    testingData = NULL;

    stopThis    = false;
}

/* Wrapper for the thread's entry method */
void NNTester::run(NNApprox* approx, NNData* data)
{
    /* Initalizes for a testing session */
    myNetwork   = approx;
    testingData = data;
    stopThis    = false;

    /* Starts the testing session */
    start();
}

/* Thread's entry method */
void NNTester::run()
{
    clearStats();

    /*Main testing loop */
    for(int i = 0; i < testingData->getNumData(); i++)
    {
        /* Stop if requested to stop */
        if(stopThis == true)
            return;

        /* Fire network and calculate the error */
        int len = 0;
        double* inputs = testingData->getData(i, &len);

        int outLen = 0;
        double* outputs = myNetwork->fire(&(inputs[0]), len, &outLen);

        double expectedValue = testingData->getOutput(i);
        double error         = getError(&(outputs[0]), outLen, expectedValue);

        /* Post test is completed */
        emit testCycleDone((((double)i)/((double)testingData->getNumData())), error, getStats());

        msleep(1);
    }

    /* Post done testing */
    emit doneTesting();
}

/* Slot that stops the testing cycle if called */
void NNTester::stopTesting()
{
    stopThis = true;
}

