#ifndef NNTrainer_H
#define NNTrainer_H

#include <QThread>
#include <cmath>

#include "NNApprox.h"
#include "../FileIO/NNData.h"
#include "../../../include/qwt_scale_draw.h"

class NNTrainer : public QThread
{
    Q_OBJECT

    protected:
       /* Network and Data */
       NNApprox* myNetwork;
       NNData*   trainingData;
       int       maxIterations;
       double    gamma;
       double    momentum;

       bool stopThis;

protected:
       virtual void clearStats() = 0;

    public slots:
       void stopTraining();

    signals:
       void doneTraining();
       void trainCycleDone(double percentDone, double error, QString* stats);

    public:
        NNTrainer();

        virtual void run() = 0;
        virtual QString* getName() = 0;
        virtual QString* getStats() = 0;

        virtual QString* getScaleName() = 0;
        virtual double getScaleMax() = 0;
        virtual QwtScaleDraw* getScaleDraw() = 0;

        void run(NNApprox* approx, NNData* data, int maxIterations, double gamma, double momentum);

};

#endif // NNTrainer_H
