/**
* Matthew Netsch
* Feb 2010
*
* DeviceProvider.cpp
*
* Header file for a thread that keeps
* track of all devices and their
* connective states.
*/

#include "DeviceProvider.h"

/* Configure */
DeviceProvider::DeviceProvider()
{
    devices    = NULL;
    testers    = NULL;

    numDevices = 0;
}

void DeviceProvider::load(NNDevice* device)
{
    numDevices++;

    devices    = (NNDevice**)realloc(devices, sizeof(NNDevice*) * numDevices);
    testers    = (DeviceTester**)realloc(testers, sizeof(DeviceTester*) * numDevices);

 
    devices[numDevices - 1] = device;
    testers[numDevices - 1] = new DeviceTester(device);
}

/* Destructor */
DeviceProvider::~DeviceProvider()
{
    if(devices != NULL)
    {
        for(int i = 0; i < numDevices; i++)
        {
            if( devices[i] != NULL )
                delete(devices[i]);
        }

        free(devices);
    }

    if(testers != NULL)
    {
        for(int i = 0; i < numDevices; i++)
        {
            if( testers[i] != NULL )
                delete(testers[i]);
        }
        free(testers);
    }


}

NNDevice** DeviceProvider::queryDevices(int* num)
{
   /* Returns a list of devices that are connected */
   NNDevice** result = (NNDevice**)malloc(sizeof(NNDevice*)*numDevices);
   int len = 0;

   for(int i = 0; i < numDevices; i++)
   {
      if((devices[i]->isConnected()))
      {
         len++;
         result[len-1] = devices[i];
      }
   }

   (*num) = len;

   return result;
}

/* Thread's entry method */
void DeviceProvider::run()
{
   //Spins and trys to update connections now and then 

   while(isRunning())
   {
      /* Go through all of the devices */
      for(int i = 0; i < numDevices; i++)
      {
          if(!isRunning())
              break;

          /* Tell its tester to test the connection */
          testers[i]->start();

          QThread::sleep(2);

          /* If the tester is still running after 2 seconds then we have lost connection */
          if(testers[i]->isRunning())
          {
              /* Terminate the tester thread and tell the device to attempt to reload its connection */
              testers[i]->terminate();
              devices[i]->setConnected(false);
              devices[i]->reload();	
          }
      }

      emit updateDevices();

   }
}
