#include "DeviceViewTab.h"

DeviceViewTab::DeviceViewTab()
{
    setLeftHeader(new QLabel(tr("Device Output:")));

    deviceChanged(NULL);
}

void DeviceViewTab::deviceChanged(NNDevice* device)
{
    currentDevice = device;

    if(currentDevice != NULL)
    {

        QTextEdit* outputView = new QTextEdit();
        outputView->setReadOnly(true);
        outputView->viewport()->setCursor(Qt::ArrowCursor);
        outputView->setWordWrapMode(QTextOption::NoWrap);

        DeviceView* view = NULL;
        QWidget* controls = new QWidget();

        currentDevice->constructViews(currentDevice, &view, &controls);

        QWidget* controlWidget = new QWidget();
        QVBoxLayout* controlLayout = new QVBoxLayout();
        controlLayout->addWidget(new QLabel(tr("Device Controls:")));
        controlLayout->addWidget(controls);
        controlWidget->setLayout(controlLayout);
        controlLayout->setSpacing(6);
        controlLayout->setContentsMargins(0, 3, 0, 0);
        controlWidget->setMaximumHeight(110);
        controlWidget->setMinimumHeight(110);

        connect(view, SIGNAL(appendConsole(QString)), outputView, SLOT(append(QString)));
        connect(view, SIGNAL(clearConsole()), outputView, SLOT(clear()));

        QSplitter* splitter = new QSplitter();
        splitter->addWidget(outputView);
        splitter->addWidget(controlWidget);

        QList<int> list;
        list.append(218);
        list.append(167);
        splitter->setSizes(list);

        splitter->setChildrenCollapsible(false);
        splitter->setOrientation(Qt::Vertical);

        update(splitter, view);

        view->setShowRefresh(true);
    }
    else
    {
        update(getDefaultView(), getDefaultView());
    }

    emit statusChanged();
}

status* DeviceViewTab::getStatus()
{

    status* result = (status*)malloc(sizeof(status));
    result->progressBar = -1.0;
    result->hintLabel = NULL;
    result->progressLabel = NULL;
    result->statusLabel = NULL;
    result->tempLabel = NULL;

    if(currentDevice == NULL)
    {
        result->hintLabel = (char*)malloc(sizeof(char) * 64);

        strcpy(result->hintLabel, " Hint: Open a device from the device menu to begin ");
    }
    else
    {
        result->statusLabel = (char*)malloc(sizeof(char) * (currentDevice->getName()->size() + 64));

        strcpy(result->statusLabel, " Device: ");
        strcat(result->statusLabel, currentDevice->getName()->toStdString().c_str());
        strcat(result->statusLabel, "  ");

    }

    return result;
}

QWidget* DeviceViewTab::getDefaultView()
{
    QTextEdit* result  = new QTextEdit();

    result->setDisabled(true);
    result->setAutoFillBackground(true);

    return result;
}
