/**
* Matthew Netsch
* Feb 2010
*
* NetworkView.h
*
* Viewport for selecting layers and nodes
* of a neural network. Contains the OpenGL
* code required to display the network.
*/

#ifndef NETWORKVIEW_H
#define NETWORKVIEW_H

#include <QGLWidget>
#include <QMessageBox>
#include <QMouseEvent>
#include <QTimer>

#include <cmath>
#include "../NeuralNetwork/NNApprox.h"

class NetworkView : public QGLWidget
{
    Q_OBJECT

    public:
      NetworkView();
     ~NetworkView();

    protected:
      void initializeGL();
      void paintEvent(QPaintEvent *event);
      void mouseReleaseEvent(QMouseEvent *event);
      void mouseMoveEvent(QMouseEvent *event);

    public slots:
     void networkUpdated(NNApprox* nn);
     void networkOpened(NNApprox* nn, QString* filename);

    private slots:
      void animate();

    signals:
      void nodeChanged(int layer, int node);

      void statusChanged();
      void changeValue(int value);

    private:

      /* Neural Network Vars */
      NNApprox*  myNetwork;
      int        currentLayer;
      int        numLayers;

      int        increment;
      GLuint     layerList;

      int        direction;
      int        currentNode;

      /* OpenGL Vars */
      double**   trans;
      double**   rot;

      double**   dTrans;
      double**   dRot;

      double***  transVals;
      double***  rotVals;

      bool       networkEnabled;

      void       startAnimation(int direction);
      void       checkNodeClick(int x, int y);

      QTimer     timer;

      int lastValue;
 };

#endif // NETWORKVIEW_H
