#include "NetworkViewTab.h"

NetworkViewTab::NetworkViewTab()
{
    table = new NetworkTable();
    view  = new NetworkView();

    connect(this, SIGNAL(networkOpened(NNApprox*,QString*)), view, SLOT(networkOpened(NNApprox*,QString*)));
    connect(this, SIGNAL(networkUpdated(NNApprox*)), view, SLOT(networkUpdated(NNApprox*)));
    connect(this, SIGNAL(networkOpened(NNApprox*,QString*)), this, SLOT(openNetwork(NNApprox*,QString*)));

    connect(view, SIGNAL(statusChanged()), this, SIGNAL(statusChanged()));

    connect(this, SIGNAL(networkOpened(NNApprox*,QString*)), table, SLOT(openNetwork(NNApprox*,QString*)));
    connect(this, SIGNAL(networkUpdated(NNApprox*)), table, SLOT(updateNetwork(NNApprox*)));
    connect(table, SIGNAL(networkUpdated(NNApprox*)), this, SIGNAL(updateNetwork(NNApprox*)));
    connect(table, SIGNAL(changeNode(int)), this, SLOT(changeNode(int)));
    connect(view, SIGNAL(nodeChanged(int,int)), table, SLOT(nodeChanged(int,int)));
    connect(view, SIGNAL(changeValue(int)), this, SLOT(changeValue(int)));

    filename = NULL;
    loadedNetwork = NULL;
    currentNode = -1;
    value = 0;

    setLeftHeader(new QLabel(tr("Node's Output Connections:")));
    construct(table, view);
}

status* NetworkViewTab::getStatus()
{

    status* result = (status*)malloc(sizeof(status));
    result->progressBar = -1.0;
    result->hintLabel = NULL;
    result->progressLabel = NULL;
    result->statusLabel = NULL;
    result->tempLabel = NULL;

    if(loadedNetwork == NULL)
    {
        result->hintLabel = (char*)malloc(sizeof(char) * 64);

        strcpy(result->hintLabel, " Hint: Open a neural network from the file menu to begin ");
    }
    else
    {
        if( currentNode < 0)
        {
            result->hintLabel = (char*)malloc(sizeof(char) * 64);
            strcpy(result->hintLabel, " Hint: Click on a node to view its outbound connections ");
        }
        else
        {
            result->statusLabel = (char*)malloc(sizeof(char) * 64);
            sprintf(result->statusLabel, " Node Selected: 0x%x ", currentNode);
        }

        result->progressLabel = (char*)malloc(sizeof(char) * (filename->size() + 64));

        strcpy(result->progressLabel, " File: ");
        strcat(result->progressLabel, filename->toStdString().c_str());
        strcat(result->progressLabel, "  ");

    }

    if( value == 0 )
    {

    }
    else if( value == -1)
    {
        result->tempLabel = (char*)malloc(sizeof(char)*64);
        strcpy(result->tempLabel, " Click for Previous layer ");
    }
    else if( value == -2 )
    {
        result->tempLabel = (char*)malloc(sizeof(char)*64);
        strcpy(result->tempLabel, " Click for Next layer ");
    }
    else
    {
        result->tempLabel = (char*)malloc(sizeof(char)*64);
        sprintf(result->tempLabel, " Click for Node 0x%x ", value);
    }

    return result;
}

void NetworkViewTab::openNetwork(NNApprox *nn, QString *filename)
{
    loadedNetwork = nn;
    this->filename = ((filename==NULL)||(filename->isEmpty())) ? (new QString("Untitled")) : new QString(filename->right(filename->size() - filename->lastIndexOf('/') - 1));

    emit statusChanged();
}

void NetworkViewTab::changedFilename(QString* filename)
{
   this->filename = ((filename==NULL)||(filename->isEmpty())) ? (new QString("Untitled")) : new QString(filename->right(filename->size() - filename->lastIndexOf('/') - 1));

   emit statusChanged();
}

void NetworkViewTab::changeNode(int node)
{
   this->currentNode = node;

   emit statusChanged();
}

void NetworkViewTab::changeValue(int value)
{
    /* 0 means ready
       1+ means node number
       -1 means previous Layer
       -2 means next Layer
    */

    this->value = value;

    emit statusChanged();
}
