#include "TestingViewTab.h"

TestingViewTab::TestingViewTab()
{
    currentTestSet = NULL;
    count = -1;

    progress = 0;
    methodName = NULL;
    numTests = 0;
    iter = 0;
    currentIter = -1;

    testing = false;

    setLeftHeader(new QLabel(tr("Testing Output:")));
    this->setAutoFillBackground(true);

    Q3TextEdit* outputView = new Q3TextEdit();
    outputView->setReadOnly(true);
    outputView->viewport()->setCursor(Qt::ArrowCursor);
    outputView->setWordWrap(Q3TextEdit::NoWrap);
    outputView->setTextFormat(Qt::LogText);
    outputView->setUndoRedoEnabled (false);
    outputView->setAutoFormatting(Q3TextEdit::AutoNone);

    TestGraph* view = new TestGraph();

    connect(this, SIGNAL(appendOutput(QString)), outputView, SLOT(append(QString)));
    connect(this, SIGNAL(clearOutput()), outputView, SLOT(clear()));

    connect(this, SIGNAL(doneIteration(double,double,QString*)), view, SLOT(testCycleDone(double,double,QString*)));
    connect(this, SIGNAL(startTest(QString*,int,int)), view, SLOT(testStarted(QString*,int,int)));

    connect(this,SIGNAL(testSetLoaded(NNData*)), view, SLOT(testSetLoaded(NNData*)));
    connect(this, SIGNAL(changeTestMethod(NNTester*)), view, SLOT(changeTestMethod(NNTester*)));

    construct(outputView, view);

}

void TestingViewTab::testSetLoaded(QString filename)
{
    count = -1;
    if(currentTestSet!=NULL)
        delete(currentTestSet);

    if((filename == NULL)||(filename.isEmpty()))
    {
        currentTestSet = NULL;

        emit statusChanged();
        return;
    }

    currentTestSet = new QString(filename.right(filename.size() - filename.lastIndexOf('/') - 1));
    emit statusChanged();
}

void TestingViewTab::animateLoading(int count)
{
   this->count = count;

   emit statusChanged();
}

status* TestingViewTab::getStatus()
{
    status* result = (status*)malloc(sizeof(status));
    result->progressBar = -1.0;
    result->hintLabel = NULL;
    result->progressLabel = NULL;
    result->statusLabel = NULL;
    result->tempLabel = NULL;

    if(methodName != NULL)
    {
        result->tempLabel = (char*)malloc(sizeof(char) * (methodName->size() + 64));

        strcpy(result->tempLabel, " Test Method: ");
        strcat(result->tempLabel, methodName->toStdString().c_str());
        strcat(result->tempLabel, " ");
     }

    if((currentTestSet == NULL) && (count < 0))
    {
        result->hintLabel = (char*)malloc(sizeof(char) * 64);

        strcpy(result->hintLabel, " Hint: Open a test set from the Test menu to begin ");
    }


   if(count == -1)
    {
       if(currentTestSet != NULL)
       {

           result->statusLabel = (char*)malloc(sizeof(char) * (currentTestSet->size() + 64));

           strcpy(result->statusLabel, " Test Set: ");
           strcat(result->statusLabel, currentTestSet->toStdString().c_str());
           strcat(result->statusLabel, "  ");
        }
   }
   else
   {
       result->statusLabel = (char*)malloc(sizeof(char) * 64);

       if(count == 0)
       {
          strcpy(result->statusLabel, " Loading .     ");
       }
       else if(count == 1)
       {
         strcpy(result->statusLabel,  " Loading   .   ");
       }
       else if(count == 2)
       {
         strcpy(result->statusLabel,  " Loading     . ");
       }
   }

   if(testing == true)
   {
        result->progressLabel = (char*)malloc(sizeof(char) * 64);
        strcpy(result->progressLabel, " Test Progress: ");

        result->progressBar = progress;
   }

    return result;
}

void TestingViewTab::testStarted(QString* methodName, int numTests, int iter)
{
    this->methodName = methodName;
    this->numTests = numTests;
    this->iter = iter;

    currentIter = -1;

    progress = 0;

    testing = true;

    QString output;
    char buf[64];
    output.append("Test Started\n");
    output.append("Using method: ");
    output.append(methodName);
    output.append("\nUsing Test Set: ");
    output.append(currentTestSet);
    output.append("\nNumber of Tests: ");
    sprintf(&(buf[0]), "%d", numTests);
    output.append(&(buf[0]));

    if(iter != 0)
    {
        output.append("\nOutput Every ");
        sprintf(&(buf[0]), "%d", iter);
        output.append(&(buf[0]));
        output.append(" Tests\n\n");
    }
    else
    {
        output.append("\nNot Outputing\n\n");
    }

    emit clearOutput();
    emit appendOutput(output);

    emit statusChanged();

    emit startTest(methodName, numTests, iter);
}

void TestingViewTab::testStopped(QString* stats)
{
    testing = false;

    QString output;
    output.append("\nTest Finished\n");

    if(stats != NULL)
    {
        output.append("\nStats:\n");
        output.append(stats);
    }

    emit appendOutput(output);

    emit statusChanged();
}

void TestingViewTab::testCycleDone(double percentDone, double error, QString* stats)
{
    progress = percentDone;

    currentIter++;

    if(iter != 0)
    {
        if((currentIter % iter) == 0)
        {
            if(stats != NULL)
            {
                QString output;
                char buf[64];
                sprintf(&(buf[0]), "\nTest %d\n", currentIter);
                output.append(&(buf[0]));

                output.append(stats);

                emit appendOutput(output);
            }
        }
    }

    emit statusChanged();

    emit doneIteration(percentDone, error, stats);

}

void TestingViewTab::changeTestMethod(QString* methodName)
{
    this->methodName = (methodName == NULL) ? NULL : new QString(*methodName);

    emit statusChanged();
}

void TestingViewTab::error(const char* title, const char* message)
{
    /* Display error message*/
    QIcon icon;
    icon.addFile(QString::fromUtf8(":/new/icons/TAC.ico"), QSize(), QIcon::Normal, QIcon::Off);

    QMessageBox msg;

    msg.setText(message);
    msg.setWindowTitle(title);
    msg.setIcon(QMessageBox::Critical);
    msg.setWindowIcon(icon);
    msg.exec();
}
