#include "TrainingViewTab.h"

TrainingViewTab::TrainingViewTab()
{
    currentTrainSet = NULL;
    count = -1;

    progress = 0;
    methodName = NULL;
    numTrain = 0;
    iter = 0;
    currentIter = -1;
    maxIterations = 0;
    gamma = 0;
    momentum = 0;

    training = false;

    setLeftHeader(new QLabel(tr("Training Output:")));
    this->setAutoFillBackground(true);

    Q3TextEdit* outputView = new Q3TextEdit();
    outputView->setReadOnly(true);
    outputView->viewport()->setCursor(Qt::ArrowCursor);
    outputView->setWordWrap(Q3TextEdit::NoWrap);
    outputView->setTextFormat(Qt::LogText);
    outputView->setUndoRedoEnabled (false);
    outputView->setAutoFormatting(Q3TextEdit::AutoNone);

    TrainGraph* view = new TrainGraph();

    connect(this, SIGNAL(appendOutput(QString)), outputView, SLOT(append(QString)));
    connect(this, SIGNAL(clearOutput()), outputView, SLOT(clear()));

    connect(this, SIGNAL(startTrain(QString*,int,int,int,double,double)), view, SLOT(trainStarted(QString*,int,int,int,double,double)));
    connect(this, SIGNAL(constantsChanged(int,int,double,double)), view, SLOT(constantsChanged(int,int,double,double)));
    connect(this, SIGNAL(changeTrainMethod(NNTrainer*)), view, SLOT(changeTrainMethod(NNTrainer*)));
    connect(this, SIGNAL(doneIteration(double,double,QString*)), view, SLOT(trainCycleDone(double,double,QString*)));

    construct(outputView, view);

}

void TrainingViewTab::trainSetLoaded(QString filename)
{
    count = -1;
    if(currentTrainSet!=NULL)
        delete(currentTrainSet);

    if((filename == NULL)||(filename.isEmpty()))
    {
        currentTrainSet = NULL;

        emit statusChanged();
        return;
    }

    currentTrainSet = new QString(filename.right(filename.size() - filename.lastIndexOf('/') - 1));
    emit statusChanged();
}

void TrainingViewTab::animateLoading(int count)
{
   this->count = count;

   emit statusChanged();
}

status* TrainingViewTab::getStatus()
{
    status* result = (status*)malloc(sizeof(status));
    result->progressBar = -1.0;
    result->hintLabel = NULL;
    result->progressLabel = NULL;
    result->statusLabel = NULL;
    result->tempLabel = NULL;

    if(methodName != NULL)
    {
        result->tempLabel = (char*)malloc(sizeof(char) * (methodName->size() + 64));

        strcpy(result->tempLabel, " Test Method: ");
        strcat(result->tempLabel, methodName->toStdString().c_str());
        strcat(result->tempLabel, " ");
     }

    if((currentTrainSet == NULL) && (count < 0))
    {
        result->hintLabel = (char*)malloc(sizeof(char) * 64);

        strcpy(result->hintLabel, " Hint: Open a training set from the Train menu to begin ");
    }


   if(count == -1)
    {
       if(currentTrainSet != NULL)
       {

           result->statusLabel = (char*)malloc(sizeof(char) * (currentTrainSet->size() + 64));

           strcpy(result->statusLabel, " Test Set: ");
           strcat(result->statusLabel, currentTrainSet->toStdString().c_str());
           strcat(result->statusLabel, "  ");
        }
   }
   else
   {
       result->statusLabel = (char*)malloc(sizeof(char) * 64);

       if(count == 0)
       {
          strcpy(result->statusLabel, " Loading .     ");
       }
       else if(count == 1)
       {
         strcpy(result->statusLabel,  " Loading   .   ");
       }
       else if(count == 2)
       {
         strcpy(result->statusLabel,  " Loading     . ");
       }
   }

   if(training == true)
   {
        result->progressLabel = (char*)malloc(sizeof(char) * 64);
        strcpy(result->progressLabel, " Training Progress: ");

        result->progressBar = progress;
   }

    return result;
}

void TrainingViewTab::trainStarted(QString* methodName, int numTrain, int iter, int maxIterations, double gamma, double momentum)
{
    this->methodName = methodName;
    this->numTrain = numTrain;
    this->iter = iter;
    this->maxIterations = maxIterations;
    this->gamma = gamma;
    this->momentum = momentum;

    currentIter = -1;

    progress = 0;

    training = true;

    QString output;
    char buf[64];
    output.append("Training Started\n");
    output.append("Using method: ");
    output.append(methodName);
    output.append("\nUsing Training Set: ");
    output.append(currentTrainSet);
    output.append("\nNumber of Training Data: ");
    sprintf(&(buf[0]), "%d", numTrain);
    output.append(&(buf[0]));
    output.append("\nMaximum Iterations: ");
    sprintf(&(buf[0]), "%d", maxIterations);
    output.append(&(buf[0]));
    output.append("\nGamma Value: ");
    sprintf(&(buf[0]), "%.20f", gamma);
    output.append(&(buf[0]));
    output.append("\nMomentum: ");
    sprintf(&(buf[0]), "%.20f", momentum);
    output.append(&(buf[0]));

    if(iter != 0)
    {
        output.append("\nOutput Every ");
        sprintf(&(buf[0]), "%d", iter);
        output.append(&(buf[0]));
        output.append(" Iterations\n\n");
    }
    else
    {
        output.append("\nNot Outputing\n\n");
    }

    emit clearOutput();
    emit appendOutput(output);

    emit statusChanged();
}

void TrainingViewTab::trainStopped(QString* stats)
{
    training = false;

    QString output;
    output.append("\nTraining Finished\n");

    if(stats != NULL)
    {
        output.append("\nStats:\n");
        output.append(stats);
    }

    emit appendOutput(output);

    emit statusChanged();
}

void TrainingViewTab::trainCycleDone(double percentDone, double error, QString* stats)
{
    progress = percentDone;

    currentIter++;

    if(iter != 0)
    {
        if((currentIter % iter) == 0)
        {
            if(stats != NULL)
            {
                QString output;
                char buf[64];
                sprintf(&(buf[0]), "\nIteration %d\n", currentIter);
                output.append(&(buf[0]));

                output.append(stats);

                emit appendOutput(output);
            }
        }
    }

    emit statusChanged();
}

void TrainingViewTab::changeTrainMethod(QString* methodName)
{
    this->methodName = (methodName == NULL) ? NULL : new QString(*methodName);

    emit statusChanged();
}

void TrainingViewTab::error(const char* title, const char* message)
{
    /* Display error message*/
    QIcon icon;
    icon.addFile(QString::fromUtf8(":/new/icons/TAC.ico"), QSize(), QIcon::Normal, QIcon::Off);

    QMessageBox msg;

    msg.setText(message);
    msg.setWindowTitle(title);
    msg.setIcon(QMessageBox::Critical);
    msg.setWindowIcon(icon);
    msg.exec();
}
