/**
* Matthew Netsch
* Feb 2010
*
* TTTDeviceSW.cpp
*
* Implements a Neural Network Device.
* This particular device is a software
* neural network for tic tac toe.
*/

#include "TTTDeviceSW.h"

/* Constructor */
TTTDeviceSW::TTTDeviceSW()
{
    connected = true;

    myNetwork = new TTTApprox();

}

/* Destructor */
TTTDeviceSW::~TTTDeviceSW()
{
    delete(myNetwork);
}

/* Re-establish everything if connection is lost */
void TTTDeviceSW::reload()
{

}

/* Retrieves the version */
int TTTDeviceSW::getVersion()
{
     return 2;
}

/* Configures the connection to the device */
void TTTDeviceSW::configure()
{

}

/* Stores the weights into the Device */
void TTTDeviceSW::store(NNApprox* approx)
{
    myNetwork->clone(approx);
}

/* Loads the weights from the device */
void TTTDeviceSW::load(NNApprox* approx)
{
  approx->clone(myNetwork);
}

/* Specifies if the device can be configured */
bool TTTDeviceSW::isConfigurable()
{
  return false;
}

/* Retrieves the connection status of the device */
bool TTTDeviceSW::isConnected()
{
   return connected;
}

/* Tests the device's connection and set the status */
void TTTDeviceSW::testConnection()
{
    myNetwork->getVersion();
    connected = true;
}

/* Set the connective status of the device */
void TTTDeviceSW::setConnected(bool connected)
{
    this->connected = connected;
}

/* Gets the name of the device for printing purposes */
QString* TTTDeviceSW::getName()
{
    return new QString("Software TTT");
}

double TTTDeviceSW::evaluate(double* inputs, int numInputs)
{
    int outLen = 0;
    double* outputs = myNetwork->fire(inputs, numInputs, &outLen);

    if( outLen <= 0 )
        return 0;

    return outputs[0];
}

void TTTDeviceSW::constructViews(NNDevice* device, DeviceView** view, QWidget** controls)
{

   TTTDeviceView* app = new TTTDeviceView();
   TTTDeviceSWControls* ops = new TTTDeviceSWControls();

   (*controls) = ops;
   (*view)     = app;

   TTTEngineThread* engine = new TTTEngineThread(device);

   ops->connect(ops, SIGNAL(highlightBoxChanged(int)), app, SLOT(highlightCheck(int)));
   ops->connect(ops, SIGNAL(newGame()), engine, SLOT(newGame()));
   ops->connect(ops, SIGNAL(OBoxChanged(int)), engine, SLOT(oCheckBox(int)));
   ops->connect(ops, SIGNAL(undoMove()), engine, SLOT(undoMove()));
   ops->connect(ops, SIGNAL(XBoxChanged(int)), engine, SLOT(xCheckBox(int)));
   ops->connect(app, SIGNAL(clicked(int,int)), engine, SLOT(clicked(int,int)));
   ops->connect(engine, SIGNAL(showWin(int,int,int,int)), app, SLOT(showWin(int,int,int,int)));
   ops->connect(engine, SIGNAL(updateBoard(BoardPosition*,double**)), app, SLOT(updateBoard(BoardPosition*,double**)));
   ops->connect(engine, SIGNAL(canUndo(bool)), ops, SLOT(canUndo(bool)));
   ops->connect(engine, SIGNAL(noWin()), app, SLOT(noWin()));

   ops->setEngine(engine);

   engine->start();
   engine->newGame();

}
