#include "BoardPosition.h"

BoardPosition::BoardPosition(int width, int height)
{
    this->width = width;
    this->height = height;
    
    pieces = (int**)malloc(sizeof(int*) * height);
    
    for(int i = 0; i < height; i++)
    {
       pieces[i] = (int*) malloc(sizeof(int) * width);
       
       for(int j = 0; j < width; j++)
          pieces[i][j] = EMPTY;
    }
}

BoardPosition::~BoardPosition()
{
   if( pieces ) 
   {
       for(int i = 0; i < height; i++)
        if(pieces[i])
          free(pieces[i]);
          
       free(pieces);
   }
}
             
void BoardPosition::reset()
{
    for(int i = 0; i < height; i++)
       for(int j = 0; j < width; j++)
          pieces[i][j] = EMPTY;
}


bool BoardPosition::addPiece(int piece, int x, int y)
{
     if ( isOutOfBounds(x, y) )
        return false;
     
     if ( getPiece(x, y) != EMPTY )
        return false;
     
     pieces[y][x] = piece;

     return true;
}

int BoardPosition::getPiece(int x, int y)
{  
     if ( isOutOfBounds(x, y) )
        return -1;
        
     return pieces[y][x];
}

void BoardPosition::movePiece(int x1, int y1, int x2, int y2)
{
     if ( getPiece(x2, y2) != EMPTY )
        return;

     addPiece( getPiece(x1, y1), x2, y2 );
     removePiece( x1, y1 );
}

void BoardPosition::removePiece( int x, int y )
{
    pieces[y][x] = EMPTY; 
}

bool BoardPosition::isOutOfBounds( int x, int y)
{
      return ((x >= width) || (x < 0) || (y >= height) || (y < 0));
}

int* BoardPosition::row( int y, int* len)
{
   (*len) = 0;

   if ( isOutOfBounds(0, y) )
        return NULL;

   (*len) = width;
   
   int* result = (int*)malloc(sizeof(int) * width );
   
   for(int i = 0; i < width; i++)
     result[i] = pieces[y][i];
     
   return result;
}

int* BoardPosition::col( int x, int* len)
{
   (*len) = 0;

   if ( isOutOfBounds(x, 0) )
        return NULL;

   (*len) = height;
   
   int* result = (int*)malloc(sizeof(int) * height);
   
   for(int i = 0; i < height; i++)
     result[i] = pieces[i][x];  
     
   return result;
}
