#include "TTTEngine.h"

TTTEngine::TTTEngine()
{
    position = new BoardPosition(W,H);
    numMoves = 0;
    moves = NULL;
    reset();
}

TTTEngine::~TTTEngine()
{
   if( position )
     delete(position);
}

int TTTEngine::getTurn()
{
    return turn;    
}

BoardPosition* TTTEngine::getPosition()
{
   return position;
}

void TTTEngine::reset()
{
     position->reset();
     turn = X;
     
     for(int i = 0; i < numMoves; i++)
        free(moves[i]);
        
     moves = (int**)realloc(moves, 0 );
     numMoves = 0;
}

void TTTEngine::move(int x, int y)
{

     if(!position->addPiece(turn, x, y))
         return;
     
     numMoves++;
     moves = (int**)realloc(moves, sizeof(int*) * numMoves);
     moves[numMoves-1] = (int*)malloc(sizeof(int) * 2 );
     moves[numMoves-1][0] = x;
     moves[numMoves-1][1] = y;
     
     nextTurn();
     
}

void TTTEngine::nextTurn()
{
     if( turn == X )
         turn = O;
     else if( turn == O )
         turn = X;
     else
         turn = X;

}
             
void TTTEngine::undo()
{
     if( numMoves <= 0 )
         return;

     int x = moves[numMoves-1][0];
     int y = moves[numMoves-1][1];
     
     free(moves[numMoves-1]);
     
     numMoves--;
     moves = (int**)realloc(moves, sizeof(int*) * numMoves);
     
     position->removePiece(x, y);
     nextTurn();
   
}

bool TTTEngine::canUndo()
{
    return (numMoves > 0);
}

void TTTEngine::view()
{
     for(int i = 0; i < H; i++)
     {
       for(int j = 0; j < W; j++)
       {
         if( position->getPiece(j, i) == X )
             printf(" X");
         else if( position->getPiece(j, i) == O )
             printf(" O");
         else
             printf("  ");
             
       }  
          printf("\n");
     }
     
     
     char state[32] = "";
     TTTState stateNum = getGameState();
     
     if( stateNum == DRAW )
         strcpy(state, "Draw");
     else if( stateNum == X_WINS )
         strcpy(state, "X Wins");
     else if( stateNum == O_WINS )
         strcpy(state, "O Wins");
     else if( stateNum == PLAYING)
         strcpy(state, "Playing");
     else
         strcpy(state, "Unknown");
     
     printf("State: %s\n", state);
     printf("Move: %s\n", (getTurn() == X ) ? "O" : "X" );
     
}

int** TTTEngine::legalMoves(int* len)
{
      int** result = NULL;
      int num = 0;
      
      for(int i = 0; i < H; i++)
      {
        for(int j = 0; j < W; j++)
        {
           if( position->getPiece(j, i) == EMPTY )
           {
               num++;
               result = (int**)realloc(result, sizeof(int*) * num);
               result[num-1] = (int*)malloc(sizeof(int) * 2);
               result[num-1][0] = j;
               result[num-1][1] = i;
           }
        }
      }
           
      (*len) = num;
      
      return result;
}

TTTState TTTEngine::pieceToGameState(int piece)
{
   if( piece == X ) 
       return X_WINS;
   
   if( piece == O )
       return O_WINS;
       
   return DRAW;
}

TTTState TTTEngine::getGameState()
{
  int piecer = 0, piecec = 0;
  int piecedr = 0, piecedl = 0;
  
  bool dontdr = false;
  bool dontdl = false;
  
  for(int i = 0; i < 3; i++)
  {
    bool dontr = false;
    bool dontc = false;
    
    if( i == 0 )
    {
        piecedr = position->getPiece(i, i);
        piecedl = position->getPiece(i, 2-i );
    }
    else
    {
        if( piecedr != position->getPiece(i, i) )
            dontdr = true;
        
        if( piecedl != position->getPiece(i, 2-i) )
            dontdl = true;
        
    }
    
    for(int j = 0; j < 3; j++)
    {
       if( j == 0 ) 
       {
         piecec = position->getPiece(i, j);
         piecer = position->getPiece(j, i);
       }
       else 
       {
          if( piecec != position->getPiece(i, j) )
          {
             dontc = true; 
          }    
          
          if( piecer != position->getPiece(j, i) )
          {
             dontr = true;
          }
       }
    }
    
    if( dontc == false )
    {
        x1 = i;
        y1 = 0;

        x2 = i;
        y2 = 2;

        if( position->getPiece(i, 0) == X )
          return X_WINS;
        else if( position->getPiece(i, 0) == O )
          return O_WINS;

    }
    
    if( dontr == false )
    {

        x1 = 0;
        y1 = i;

        x2 = 2;
        y2 = i;

       if( position->getPiece(0, i) == X )
          return X_WINS;  
        else if( position->getPiece(0, i) == O )
          return O_WINS;

    }
    
  }
  
    if( dontdr == false )
    {

        x1 = 0;
        y1 = 0;

        x2 = 2;
        y2 = 2;

        if( position->getPiece(1, 1) == X )
          return X_WINS;
        else if( position->getPiece(1, 1) == O )
          return O_WINS;

    }
    
    if( dontdl == false )
    {
        x1 = 0;
        y1 = 2;

        x2 = 2;
        y2 = 0;

         if( position->getPiece(1,1) == X )
          return X_WINS;
        else if( position->getPiece(1, 1) == O )
          return O_WINS;

    }
  
  if( numMoves == 9 )
  {
      return DRAW;
  }
      
  return PLAYING;
}

int TTTEngine::getX1()
{
    return x1;
}

int TTTEngine::getY1()
{
    return y1;
}

int TTTEngine::getX2()
{
    return x2;
}

int TTTEngine::getY2()
{
    return y2;
}
