#include "TTTEngineThread.h"

TTTEngineThread::TTTEngineThread(NNDevice* device)
{
   this->device = device;
   engine = NULL;

   x = false;
   o = false;

   timer.setSingleShot(true);
   connect(&timer, SIGNAL(timeout()), this, SLOT(computerClick()));

   running = false;

}

void TTTEngineThread::newGame()
{
    if(engine != NULL)
    {
        delete(engine);
    }

    engine = new TTTEngine();

    int nextTurn = engine->getTurn();

    if(((nextTurn == X)&&(this->x))||((nextTurn == O)&&(this->o)))
    {
        timer.start( (rand() % 1000) + 1000 );
    }

    update();

}

void TTTEngineThread::update()
{
    double** weights = getWeights();

    emit updateBoard(engine->getPosition(), weights);

    if((engine->getGameState()==PLAYING)||(engine->getGameState()==DRAW))
    {
        emit noWin();
    }

    free(weights);

    emit canUndo(engine->canUndo());
}

double** TTTEngineThread::getWeights()
{
    double** result = (double**)malloc(sizeof(double*) * 3);
    for(int i = 0; i < 3; i++)
        result[i] = (double*)malloc(sizeof(double)*3);

    BoardPosition* position = engine->getPosition();

    for(int i = 0; i < 3; i++)
    {
        for(int j = 0; j < 3; j++)
        {
            if(position->getPiece(j, i) == 0)
            {
                engine->move(j, i);

                int len = 0;
                double* inputs = position_to_array(engine->getPosition(), &len);

                result[j][i] = device->evaluate(inputs, len);
                engine->undo();
                free(inputs);
            }
            else
            {
                result[j][i] = -100;
            }
        }
    }

    return result;
}

void TTTEngineThread::undoMove()
{
    engine->undo();

    update();
}

void TTTEngineThread::xCheckBox(int state)
{
    this->x = (state == Qt::Checked);

    if( (engine->getTurn() == X) && (x == true) )
    {
        timer.start((rand() % 1000) + 1000);
    }
}

void TTTEngineThread::oCheckBox(int state)
{
    this->o = (state == Qt::Checked);

    if( (engine->getTurn() == O) && (o == true) )
    {
        timer.start((rand() % 1000) + 1000);
    }
}

void TTTEngineThread::clicked(int x, int y)
{
    if(engine->getGameState()!=PLAYING)
        return;

    if( (x == -2) || (y == -2) )
        return;

    int turn = engine->getTurn();

    if(((turn == X)&&(this->x))||((turn == O)&&(this->o)))
    {  /* Computer Move */

        if( (x != -1) || (y != -1) )
            return;

        double** weights = getWeights();

        double greatest = -100;

        for(int i = 0; i < 3; i++)
        {
            for(int j = 0; j < 3; j++)
            {
               if(weights[j][i] > greatest)
               {
                    greatest = weights[j][i];

                    x = j;
                    y = i;
               }
            }
        }

        free(weights);
    }

    engine->move(x, y);

    update();

    if(engine->getGameState()!=PLAYING)
    {

        if(engine->getGameState() != DRAW)
        {
            emit showWin(engine->getX1(), engine->getY1(), engine->getX2(), engine->getY2());
        }

        emit gameFinished(engine->getGameState());
    }
    else
    {
        int nextTurn = engine->getTurn();

        if(((nextTurn == X)&&(this->x))||((nextTurn == O)&&(this->o)))
        {
            timer.start( (rand() % 1000) + 1000 );
        }
    }

}

void TTTEngineThread::computerClick()
{
    int turn = engine->getTurn();

    if(((turn == X)&&(this->x))||((turn == O)&&(this->o)))
    {
         clicked(-1, -1);
    }
}

double* TTTEngineThread::position_to_array(BoardPosition* position, int* len)
{
    int newLen = 18;

     (*len) = newLen;

     double* result = (double*)malloc(sizeof(double) * newLen);

     int index = 0;

     for(int i = 0; i < 3; i++)
     {
      for(int j = 0; j < 3; j++)
      {
         int piece = position->getPiece(i, j);

         if( piece == 0 )
         {
             result[index] = 0.2; index++;
             result[index] = 0.2; index++;
         }else if( piece == X )
         {
             result[index] = 0.8; index++;
             result[index] = 0.2; index++;
         }else if( piece == O )
         {
             result[index] = 0.2; index++;
             result[index] = 0.8; index++;
         }
      }
     }

     return result;
}

void TTTEngineThread::stop()
{
  running = false;
}

void TTTEngineThread::run()
{
  running = true;

  while( running );

}

