#include "RMSTester.h"

RMSTester::RMSTester()
{
    errors = NULL;
    numErrors = 0;
}

double RMSTester::getError(double* outputs, int outLen, double expectedOutput)
{
   double error =  (expectedOutput - outputs[0]) * (expectedOutput - outputs[0]);

   numErrors++;
   errors = (double*)realloc(errors, sizeof(double) * numErrors);
   errors[numErrors - 1] = std::abs(expectedOutput - outputs[0]);

   return error;
}

QString* RMSTester::getName()
{
    return new QString("Root Mean Square");
}


QString* RMSTester::getStats()
{

        if(numErrors <= 0)
            return NULL;

        double average  = 0;
        double rms = 0;
        double maxError = -1.0;

        for(int i = 0; i < numErrors; i++)
        {
            average += errors[i];
            rms += errors[i] * errors[i];

            maxError = (errors[i] > maxError) ? errors[i] : maxError;
        }

        average = average / numErrors;
        rms = sqrt(rms);

        double std = 0;

        for(int i = 0; i < numErrors; i++)
        {
            std += (errors[i] - average) * (errors[i] - average);
        }

        if(numErrors > 1)
            std = sqrt((std / (numErrors - 1)));
        else
            std = 0;

        QString* output = new QString("");

        char buf[512];
        output->append("Mean:\t\t");

        sprintf(&(buf[0]), "%.20f\n", average);
        output->append(&(buf[0]));

        output->append("Std Deviation:\t");

        sprintf(&(buf[0]), "%.20f\n", std);
        output->append(&(buf[0]));

        output->append("Error (RMS):\t");

        sprintf(&(buf[0]), "%.20f\n", rms);
        output->append(&(buf[0]));

        output->append("Max Error:\t");

        sprintf(&(buf[0]), "%.20f\n", maxError);
        output->append(&(buf[0]));

        return output;

}

void RMSTester::clearStats()
{
    if(errors != NULL)
        free(errors);

    errors = NULL;
    numErrors = 0;
}

QString* RMSTester::getScaleName()
{
    return new QString("Error (Root Mean Square)");
}

double RMSTester::getScaleMax()
{
    return 1.0;
}

QwtScaleDraw* RMSTester::getScaleDraw()
{
    return NULL;
}
