#include "WLDTester.h"


class WLDScaleDraw: public QwtScaleDraw
{
public:
    virtual QwtText label(double v) const
    {
        char buf[64];
        sprintf(&(buf[0]), "%.2f", v * 3);

        return QwtText(QString(&(buf[0])), QwtText::AutoText );
    }
};

WLDTester::WLDTester()
{
    errors = NULL;
    numErrors = 0;
}

double WLDTester::getError(double* outputs, int outLen, double expectedOutput)
{
   double error = expectedOutput - outputs[0];

   numErrors++;
   errors = (double*)realloc(errors, sizeof(double) * numErrors);
   errors[numErrors - 1] = std::abs(error);

   if( (error <= 1.0/6.0)&&(error >= -1.0/6.0) )
   { 
      error = 0.0;
   }
   else if ( abs(error) <= 0.5 )
   {
      error = 0.5;
   }
   else
   {
      error = 1.0;
   }

   return error;

}

QString* WLDTester::getName()
{
    return new QString("Win, Loss, Draw");
}

QString* WLDTester::getStats()
{
    if(numErrors <= 0)
        return NULL;

    double average  = 0;
    double rms = 0;
    double maxError = -1.0;
    int numCorrect  = 0;

    for(int i = 0; i < numErrors; i++)
    {
        average += errors[i];
        rms += errors[i] * errors[i];

        if(errors[i] <= (1.0/6.0))
            numCorrect++;

        maxError = (errors[i] > maxError) ? errors[i] : maxError;
    }

    average = average / numErrors;
    rms = sqrt(rms);

    double std = 0;


    for(int i = 0; i < numErrors; i++)
    {
        std += (errors[i] - average) * (errors[i] - average);
    }

    if(numErrors > 1)
        std = sqrt((std / (numErrors - 1)));
    else
        std = 0;

    QString* output = new QString("");

    char buf[512];
    output->append("Mean:\t\t");

    sprintf(&(buf[0]), "%.20f\n", average);
    output->append(&(buf[0]));

    output->append("Std Deviation:\t");

    sprintf(&(buf[0]), "%.20f\n", std);
    output->append(&(buf[0]));

    output->append("Error (RMS):\t");

    sprintf(&(buf[0]), "%.20f\n", rms);
    output->append(&(buf[0]));

    output->append("Max Error:\t");

    sprintf(&(buf[0]), "%.20f\n", maxError);
    output->append(&(buf[0]));

    output->append("Number Correct:\t");

    sprintf(&(buf[0]), "%d / %d\n", numCorrect, numErrors);
    output->append(&(buf[0]));

    return output;
}


void WLDTester::clearStats()
{
    if(errors != NULL)
        free(errors);

    errors = NULL;
    numErrors = 0;
}

QString* WLDTester::getScaleName()
{
    return new QString("Number Incorrect");
}

double WLDTester::getScaleMax()
{
    return 1.0;
}

QwtScaleDraw* WLDTester::getScaleDraw()
{
    return new WLDScaleDraw();
}

