#include "SOITrainer.h"

SOITrainer::SOITrainer()
{
    errors = NULL;
    numErrors = 0;
}

void SOITrainer::run()
{
   myNetwork->setGamma(gamma);
   myNetwork->setMomentum(momentum);

   clearStats();
   errors = (double*)malloc(sizeof(double) * trainingData->getNumData());
   numErrors = trainingData->getNumData();

   for(int i = 0; i < maxIterations; i++)
   {

       int index = 0;
       double total_error = 0;

       for(int j = 0; j < trainingData->getNumData(); j++)
       {
            if(stopThis == true)
                break;

            int dataLen = 0;
            double* inputs = trainingData->getData(j, &dataLen);

            int out = 0;
            trainingData->getOutput(j) - (myNetwork->fire(&(inputs[0]), dataLen, &out))[0];

            double expected[] = {trainingData->getOutput(j)};
            myNetwork->train(&(expected[0]), 1);

       }

       index = 0;
       for(int j = 0; j < trainingData->getNumData(); j++)
       {
            int dataLen = 0;
            double* inputs = trainingData->getData(j, &dataLen);

            int out = 0;
            double error = trainingData->getOutput(j) - (myNetwork->fire(&(inputs[0]), dataLen, &out))[0];

            errors[index] = std::abs(error);
            index++;

            total_error += error*error;
       }

       if(stopThis == true)
           break;

       emit trainCycleDone( (((double)i) / ((double)maxIterations)), sqrt(total_error/trainingData->getNumData()), getStats());
       msleep(1);

   }

   emit doneTraining();

}

QString* SOITrainer::getName()
{
    return new QString("Set Over Iterations");
}


QString* SOITrainer::getStats()
{
        if(numErrors <= 0)
            return NULL;

        double average  = 0;
        double rms = 0;
        double maxError = -1.0;
        int numCorrect = 0;

        for(int i = 0; i < numErrors; i++)
        {
            average += errors[i];
            rms += errors[i] * errors[i];

	   if(errors[i] <= (1.0/6.0))
        	    numCorrect++;

            maxError = (errors[i] > maxError) ? errors[i] : maxError;
        }

        average = average / numErrors;
        rms = sqrt(rms);

        double std = 0;

        for(int i = 0; i < numErrors; i++)
        {
            std += (errors[i] - average) * (errors[i] - average);
        }

        if(numErrors > 1)
            std = sqrt((std / (numErrors - 1)));
        else
            std = 0;

        QString* output = new QString("");

        char buf[512];
        output->append("Mean:\t\t");

        sprintf(&(buf[0]), "%.20f\n", average);
        output->append(&(buf[0]));

        output->append("Std Deviation:\t");

        sprintf(&(buf[0]), "%.20f\n", std);
        output->append(&(buf[0]));

        output->append("Error (RMS):\t");

        sprintf(&(buf[0]), "%.20f\n", rms);
        output->append(&(buf[0]));

        output->append("Max Error:\t");

        sprintf(&(buf[0]), "%.20f\n", maxError);
        output->append(&(buf[0]));

	output->append("Number Correct:\t");

	sprintf(&(buf[0]), "%d / %d\n", numCorrect, numErrors);
	output->append(&(buf[0]));

        return output;

}

void SOITrainer::clearStats()
{
    if(errors != NULL)
        free(errors);

    errors = NULL;
    numErrors = 0;
}


QString* SOITrainer::getScaleName()
{
  return new QString("Error (Root Mean Square)");
}

double SOITrainer::getScaleMax()
{
    return 0.4;
}

QwtScaleDraw* SOITrainer::getScaleDraw()
{
    return NULL;
}
