/**
* Matthew Netsch
* Feb 2010
*
* TTTDeviceView.cpp
*
* Contains the signals and widgets
* needed to display the error rate
* group box for the tester.
*/

#include "TTTDeviceView.h"

/* Constructor */
TTTDeviceView::TTTDeviceView()
{
    /* Initalize */
    weights   = (double**)malloc(sizeof(double*) *  3);

    for(int i = 0; i < 3; i++)
    {
        weights[i] = (double*)malloc(sizeof(double) * 3);
    }

    position = NULL;

    highlight = false;

    /* Setup Widgets */
    scene = new QGraphicsScene();

    scene->setSceneRect(0, 0, width(), height());

    setScene(scene);
    setSceneRect(0, 0, width(), height());

    setMouseTracking ( true );

    items     = (QGraphicsItem**)malloc(sizeof(QGraphicsItem*) * 32);
    itemCount = 0;

    winLine   = NULL;

    showRefresh = false;

    lastx1 = 0; lasty1 = 0;
    lastx2 = 0; lasty2 = 0;

    selfUpdate = false;

}

/* Clears the graph */
void TTTDeviceView::destroyItems()
{
  if(items != NULL)
  {
    for(int i = 0; i < itemCount; i++)
    {
        scene->removeItem(items[i]);
    }

    itemCount    = 0;

    /* Display */
    if(showRefresh == true)
    show();
  }
}

void TTTDeviceView::resizeEvent(QResizeEvent *event)
{

    scene->setSceneRect(0, 0, width(), height());

    updateBoard(position, weights);

    if(winLine != NULL)
    {
        scene->removeItem(winLine);
        winLine = NULL;
        showWin(lastx1, lasty1, lastx2, lasty2);
    }
}

/* SLOT that gets called when a test cycle has been completed */
void TTTDeviceView::updateBoard(BoardPosition* position, double** weights)
{
   if(position == NULL)
     return;

   destroyItems();

   double scalex = width()  / 500.0;
   double scaley = height() / 260.0;

   double transx = 375.0 * ( (width()/750.0)  - scalex );

   /* Copy Weights */
   double high = -10000;
   double low  = 10000;
   QString* message = new QString("Evaluation from Neural Network\n");
   char buf[64];

   for(int i = 0; i < 3; i++)
   {

       message->append("\n");

       for(int j = 0;  j < 3; j++)
       {
           this->weights[j][i] = weights[j][i];

           if(weights[j][i] < -10)
           {
               sprintf(&(buf[0]), "N/A\t");
               message->append(&(buf[0]));

               continue;
           }

           sprintf(&(buf[0]), "%f\t", weights[j][i]);
           message->append(&(buf[0]));

           high = (weights[j][i] > high) ? weights[j][i] : high;
           low  = (weights[j][i] < low) ? weights[j][i] : low;
       }
   }


   this->position = position;

   int xCoord = 254 * scalex + transx;
   int yCoord = 7 * scaley;

   QLinearGradient grad = QLinearGradient(0, 0, 4, 0);
   grad.setColorAt(0, QColor(0, 0, 0, 200));
   grad.setColorAt(0.25, QColor(64, 64, 64, 200));
   grad.setColorAt(0.75, QColor(230, 230, 230, 200));
   grad.setColorAt(1, QColor(180, 180, 180, 200));

   QBrush brush = QBrush(grad);
   QPen   pen   = QPen(brush, 7, Qt::SolidLine, Qt::RoundCap);

   QLinearGradient grad2 = QLinearGradient(0, 0, 0, 4);
   grad2.setColorAt(1, QColor(0, 0, 0, 200));
   grad2.setColorAt(0.75, QColor(64, 64, 64, 200));
   grad2.setColorAt(0.25, QColor(230, 230, 230, 200));
   grad2.setColorAt(0, QColor(180, 180, 180, 200));

   QBrush brush2 = QBrush(grad2);
   QPen   pen2   = QPen(brush2, 7, Qt::SolidLine, Qt::RoundCap);

   QGraphicsLineItem* item = scene->addLine(0, 0, 232 * scalex, 0, pen2);
   item->setPos(258 * scalex + transx, 85 * scaley);
   itemCount++;
   items[itemCount-1] = item;

   item = scene->addLine(0, 0, 232 * scalex, 0, pen2);
   item->setPos(258 * scalex + transx, 170 * scaley);
   itemCount++;
   items[itemCount-1] = item;

   item = scene->addLine(0, 0, 0, 232 * scaley, pen);
   item->setPos(333*scalex + transx, 12*scaley);
   itemCount++;
   items[itemCount-1] = item;

   item = scene->addLine(0, 0, 0, 232*scaley, pen);
   item->setPos(416*scalex + transx, 12*scaley);
   itemCount++;
   items[itemCount-1] =  item;

   for(int i = 0; i < 3; i++)
   {
       for(int j = 0; j < 3; j++)
       {
           QBrush brush;
           QPen   pen;

           switch(position->getPiece(j, i))
           {
           case 1: /* Print X */

               brush = QBrush(QColor(0, 0, 0, 200));
               pen   = QPen(brush, 3, Qt::SolidLine, Qt::RoundCap);

               itemCount++;
               items[itemCount-1] = scene->addLine(xCoord+5 * scalex, yCoord+5*scaley, xCoord+70 * scalex, yCoord+70*scaley, pen);

               itemCount++;
               items[itemCount-1] = scene->addLine(xCoord+5 * scalex, yCoord+70 * scaley, xCoord+70 * scalex, yCoord+5 * scaley, pen);

               break;
           case 2: /* Print O */

               brush = QBrush(QColor(60, 60, 60, 200));
               pen   = QPen(brush, 3, Qt::SolidLine, Qt::RoundCap);

               itemCount++;
               items[itemCount-1] = scene->addEllipse(xCoord+5*scalex, yCoord+5*scaley, 65 * scalex, 65 * scaley, pen);

               break;
           default: /* Print Highlight */
               if(highlight)
               {
                  if(high != low)
                  {

                    double scale  = 255 * (weights[j][i] - low) / (high - low);
                    int    red    = 255 - scale;
                    int    green  = scale;

                    brush = QBrush(QColor(red, green, 0, 200));
                    pen   = QPen(brush, 1, Qt::SolidLine, Qt::RoundCap);

                    itemCount++;
                    items[itemCount-1] = scene->addRect(xCoord+5*scalex, yCoord+5*scaley, 65*scalex, 65*scaley, pen, brush);
                  }
               }
           }

           xCoord += 83 * scalex;
       }
       xCoord  = 254 * scalex + transx;
       yCoord += 83 * scaley;
   }

   scene->setSceneRect(0, 0, width(), height());

   /* Update the graph and prepare for the next set of points */
   if(showRefresh == true)
    show();

   message->append("\n\n");

   emit clearConsole();
   emit appendConsole(*message);

}

void TTTDeviceView::setShowRefresh(bool val)
{
    showRefresh = val;

    updateBoard(position, weights);
}

void TTTDeviceView::showWin(int x1, int y1, int x2, int y2)
{

    double scalex = width()  / 500.0;
    double scaley = height() / 260.0;

    double transx = 375.0 * ( (width()/750.0)  - scalex );

     int xCoord1 = ((x1 * 83) + 291) * scalex + transx;
     int xCoord2 = ((x2 * 83) + 291) * scalex + transx;
     int yCoord1 = ((y1 * 80) + 49) * scaley;
     int yCoord2 = ((y2 * 80) + 49) * scaley;

     lastx1 = x1; lasty1 = y1;
     lastx2 = x2; lasty2 = y2;

     QBrush brush = QBrush(QColor(0, 0, 255, 200));
     QPen   pen   = QPen(brush, 3, Qt::SolidLine, Qt::RoundCap);

     winLine = scene->addLine(xCoord1, yCoord1, xCoord2, yCoord2, pen);

}

void TTTDeviceView::noWin()
{
    if(winLine != NULL)
    {
         scene->removeItem(winLine);
         winLine = NULL;
    }
}

void TTTDeviceView::highlightCheck(int status)
{
     highlight = (status == Qt::Checked);

     updateBoard(position, weights);

}

void TTTDeviceView::mouseMoveEvent ( QMouseEvent * event )
{
    double scalex = width()  / 500.0;
    double scaley = height() / 260.0;

    double transx = 375.0 * ( (width()/750.0)  - scalex ) + 10;

    /* Get position of click */
    int xCoord = event->x();
    int yCoord = event->y();

    int x = -2;
    int y = -2;

    if((xCoord < (341 * scalex + transx)) &&(xCoord > (258 * scalex + transx)))
        x = 0;
    else if((xCoord > (424 * scalex + transx) ) && (xCoord < (507 * scalex + transx)))
        x = 2;
    else if((xCoord > (341 * scalex + transx)) &&(xCoord < (424 * scalex + transx)))
        x = 1;

    if((yCoord < (89 * scaley)) && (yCoord > (12 * scaley)))
        y = 0;
    else if((yCoord > (166 * scaley)) && (yCoord < (scaley * 243)))
        y = 2;
    else if((yCoord > (89 * scaley)) && (yCoord < (scaley * 166)))
        y = 1;

    if((x == -2) || (y == -2))
    {
        setCursor(Qt::ArrowCursor);
        return;
    }

    setCursor(Qt::PointingHandCursor);

}


/* Event that gets called when the mouse is released */
void TTTDeviceView::mouseReleaseEvent ( QMouseEvent * event )
{

    double scalex = width()  / 500.0;
    double scaley = height() / 260.0;

    double transx = 375.0 * ( (width()/750.0)  - scalex ) + 10;

    /* Get position of click */
    int xCoord = event->x();
    int yCoord = event->y();

    int x = -2;
    int y = -2;

    if((xCoord < (341 * scalex + transx)) &&(xCoord > (258 * scalex + transx)))
        x = 0;
    else if((xCoord > (424 * scalex + transx) ) && (xCoord < (507 * scalex + transx)))
        x = 2;
    else if((xCoord > (341 * scalex + transx)) &&(xCoord < (424 * scalex + transx)))
        x = 1;

    if((yCoord < (89 * scaley)) && (yCoord > (12 * scaley)))
        y = 0;
    else if((yCoord > (166 * scaley)) && (yCoord < (scaley * 243)))
        y = 2;
    else if((yCoord > (89 * scaley)) && (yCoord < (scaley * 166)))
        y = 1;

    emit clicked(x, y);

}
