/**
* Matthew Netsch
* Feb 2010
*
* main.cpp
*
* Main function that constructs the
* main window and starts its execution.
*/

#include <QApplication>
#include <cstdlib>
#include <ctime>

#include "MainWindow.h"
#include "UserFiles/TTTFiles/TTTServiceLocator.h"
#include "UserFiles/TTTFiles/Devices/TTTDeviceSW.h"
#include "UserFiles/TTTFiles/Testers/RMSTester.h"
#include "UserFiles/TTTFiles/Testers/WLDTester.h"
#include "UserFiles/TTTFiles/Trainers/SOITrainer.h"

int main(int argc, char *argv[])
{
    QApplication app(argc, argv);

    srand(time(NULL));

    /* Set up providers for application */
    DeviceProvider* deviceProvider = new DeviceProvider();
    deviceProvider->load(new TTTDeviceSW());

    TestProvider*  testProvider = new TestProvider();
    testProvider->load(new RMSTester());
    testProvider->load(new WLDTester());

    TrainerProvider* trainerProvider = new TrainerProvider();
    trainerProvider->load(new SOITrainer());

    /* Set up service locator */
    TTTServiceLocator* myServiceLocator = new TTTServiceLocator();
    myServiceLocator->setDeviceProvider(deviceProvider);
    myServiceLocator->setTestProvider(testProvider);
    myServiceLocator->setTrainerProvider(trainerProvider);

    ServiceLocator::load(myServiceLocator);

    MainWindow wind;
    wind.show();

    return app.exec();
}
